import time
from typing import Tuple
import unittest
import datetime
import pandas as pd
import uuid
import random

from .testcredentials import TEST_IDENTITY
from dhi.platform import metadata, transfer, timeseries
from dhi.platform.authentication import ApiKeyIdentity
from dhi.platform.base.exceptions import MikeCloudRestApiException
from dhi.platform.commonmodels import AttributeDataType, AttributeOperator, ItemDefinition, PropertyDataType, PropertyDefinition, QueryCondition, TimeSeriesDataType
import os

class TestTransferTest(unittest.TestCase):

    _verbosity = 0
    _project_id = None
    _identity = None
    _test_data_dir = None
    
    def setUp(self) -> None:
        if not self._identity:
            self._identity = TEST_IDENTITY
        
        self._tsclient = timeseries.TimeSeriesClient(
            verbose=self._verbosity, 
            identity=self._identity
        )

        self._metadataclient = metadata.MetadataClient(verbose=self._verbosity, identity=TEST_IDENTITY)

        self._test_data_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
        self._stamp = stamp = datetime.datetime.utcnow().strftime('%Y%m%d%H%M%S')
        if not self._project_id:
            name = 'Python test ' + self._stamp
            projectInput = metadata.CreateProjectInput(name, 'Project created by Python SDK test')
            project = self._metadataclient.create_project(projectInput)
            self._project_id = project.id

    def tearDown(self) -> None:
        super().tearDown()
        if self._project_id:
            self._metadataclient.delete_project(self._project_id, permanently=True)
        

    def _assert_item_definition(self, expected:ItemDefinition, actual:ItemDefinition):
        self.assertIsNotNone(actual)
        self.assertEqual(expected.name, actual.name)
        self.assertEqual(expected.data_type, actual.data_type)
        self.assertEqual(expected.item, actual.item)
        self.assertEqual(expected.unit, actual.unit)
        self.assertEqual(expected.timeseries_data_type, actual.timeseries_data_type)

    def _assert_equal_ts_properties(self, expected:Tuple[PropertyDefinition], actual:Tuple[PropertyDefinition]):
        expected = [p.body() for p in expected]
        actual = [p.body() for p in actual]
        self.assertEqual(expected, actual)
    
    def _assert_equal_ts_datafields(self, expected:Tuple[timeseries.DataFieldDefinition], actual:Tuple[timeseries.DataFieldDefinition]):
        expected = [f.body() for f in expected]
        actual = [f.body() for f in actual]
        self.assertEqual(expected, actual)

    def _assert_time_equal_to_seconds(self, a:datetime.datetime, b:datetime.datetime):
        are_equal = a.date() == b.date() and a.hour == b.hour and a.minute == b.minute and a.second == b.second
        self.assertTrue(are_equal)
    
    def test_create_timeseries_dataset_with_properties_is_ok(self):
        
        ts_schema = (PropertyDefinition("propertydefinitionname", PropertyDataType.TEXT),)
        name = f"Py TS {self._stamp}"
        description = "TimeSeries dataset created from Python"
        metadata = {"metadatakey1": "metadataentry1"}
        properties = {"propertykey1": "propertyentry1"}
        
        input = timeseries.TimeSeriesDatasetInput(
            name,
            description,
            ts_schema,
            metadata,
            properties
        )

        ts_dataset = self._tsclient.create_timeseries_dataset(self._project_id, input)
        self.assertIsNotNone(ts_dataset)
        self.assertTrue(ts_dataset.id)
        self._assert_equal_ts_properties(ts_dataset.timeseries_properties, ts_schema)
        self.assertEqual(ts_dataset.metadata, metadata)
        self.assertIsNotNone(ts_dataset.items)
        self.assertFalse(ts_dataset.items)

        tsd = self._tsclient.get_timeseries_dataset(self._project_id, ts_dataset.id)
        self.assertIsNotNone(tsd)
        self.assertEqual(tsd.id, ts_dataset.id)
        self._assert_equal_ts_properties(tsd.timeseries_properties, ts_schema)
        self.assertEqual(tsd.metadata, metadata)
        self.assertIsNotNone(tsd.items)
        self.assertFalse(tsd.items)
    
    def test_create_timeseries_dataset_with_properties_from_schema_is_ok(self):
        
        ts_schema = (PropertyDefinition("propertydefinitionname", PropertyDataType.TEXT),)
        name = f"Py TS {self._stamp}"
        description = "TimeSeries dataset created from Python"

        ts_dataset = self._tsclient.create_timeseries_dataset_from_schema(self._project_id, name, description, ts_schema)
        self.assertIsNotNone(ts_dataset)
        self.assertTrue(ts_dataset.id)
        self._assert_equal_ts_properties(ts_dataset.timeseries_properties, ts_schema)
        self.assertEqual(ts_dataset.metadata, {})
        self.assertIsNotNone(ts_dataset.items)
        self.assertFalse(ts_dataset.items)

        tsd = self._tsclient.get_timeseries_dataset(self._project_id, ts_dataset.id)
        self.assertIsNotNone(tsd)
        self.assertEqual(tsd.id, ts_dataset.id)
        self._assert_equal_ts_properties(tsd.timeseries_properties, ts_schema)
        self.assertEqual(tsd.metadata, {})
        self.assertIsNotNone(tsd.items)
        self.assertFalse(tsd.items)

    def test_create_dataset_with_data_is_ok(self):
        ts_dataset = self._tsclient.create_timeseries_dataset_from_schema(self._project_id, "Py TS with data")
        
        self.assertIsNotNone(ts_dataset)
        self.assertTrue(ts_dataset.id)

        time.sleep(2) # let events propagate in local development

        tds = self._tsclient.get_timeseries_dataset(self._project_id, ts_dataset.id)

        self.assertIsNotNone(tds)
        self.assertEqual(ts_dataset.id, tds.id)

        item = ItemDefinition(
            "Temperature", 
            metadata.UnitId.EUMUDEGREECELSIUS,
            metadata.ItemId.EUMITEMPERATURE,
            AttributeDataType.SINGLE,
            TimeSeriesDataType.MEAN_STEP_BACKWARD
        )

        ts = self._tsclient.add_timeseries(self._project_id, tds.id, item)

        self.assertIsNotNone(ts)
        self.assertTrue(ts.id)
        self._assert_item_definition(item, ts.item)

        ts2 = self._tsclient.get_timeseries(self._project_id, tds.id, ts.id)
        
        self.assertIsNotNone(ts2)
        self.assertEqual(ts2.id, ts.id)
        self._assert_item_definition(ts.item, ts2.item)

        today = datetime.datetime.today()
        # add time series values
        data = pd.DataFrame([float(12.34)], index=[today])
        success = self._tsclient.add_timeseries_values(self._project_id, tds.id, ts.id, data)
        self.assertTrue(success)

        # get time series values
        tsv = self._tsclient.get_timeseries_values(self._project_id, tds.id, ts.id)

        self.assertIsNotNone(tsv)
        self.assertEqual(data.shape, tsv.shape)
        self._assert_time_equal_to_seconds(data.index[0], tsv.index[0])
        self.assertEqual(data[0][0], tsv["values"][0])

        # add more time series values
        data2 = pd.DataFrame([float(13.45)], index=[today - datetime.timedelta(hours=-1)])
        success = self._tsclient.add_timeseries_values(self._project_id, tds.id, ts.id, data2)

        self.assertTrue(success)

        # get timeseries values again
        tsv2 = self._tsclient.get_timeseries_values(self._project_id, tds.id, ts.id)

        self.assertIsNotNone(tsv2)
        self.assertEqual((2, 1), tsv2.shape)
        self._assert_time_equal_to_seconds(data.index[0], tsv2.index[0])
        self.assertEqual(data[0][0], tsv2["values"][0])
        self.assertEqual(data2[0][0], tsv2["values"][1])

        # list time series
        time_series_list = [*self._tsclient.list_timeseries(self._project_id, tds.id)]

        self.assertEqual(len(time_series_list), 1)
        self.assertTrue(time_series_list[0].id, ts.id)

    def test_query_time_series_is_ok(self):
        tds = self._tsclient.create_timeseries_dataset_from_schema(self._project_id, "Py Query TS")
        self.assertIsNotNone(tds)
        self.assertTrue(tds.id)

        # add time series
        item1 = ItemDefinition(
            "Temperature",
            metadata.UnitId.EUMUDEGREECELSIUS,
            metadata.ItemId.EUMITEMPERATURE,
            data_type=AttributeDataType.SINGLE
        )

        ts1 = self._tsclient.add_timeseries(self._project_id, tds.id, item1)

        self.assertIsNotNone(item1)
        self._assert_item_definition(item1, ts1.item)

        # add timeseries values
        data1 = pd.DataFrame([float(12.34)], index=[datetime.datetime.utcnow()])
        self._tsclient.add_timeseries_values(self._project_id, tds.id, ts1.id, data1)
        data2 = pd.DataFrame([float(16.8)], index=[datetime.datetime.utcnow() + datetime.timedelta(minutes=1)])
        self._tsclient.add_timeseries_values(self._project_id, tds.id, ts1.id, data2)

        # add time series
        item2 = ItemDefinition(
            "Rainfall",
            metadata.UnitId.EUMUPERMILLILITER,
            metadata.ItemId.EUMIRAINFALL,
            data_type=AttributeDataType.SINGLE
        )

        ts2 = self._tsclient.add_timeseries(self._project_id, tds.id, item2)

        self.assertIsNotNone(ts2)
        self._assert_item_definition(item2, ts2.item)

        # add timeseries values
        data1 = pd.DataFrame([float(6.9)], index=[datetime.datetime.utcnow()])
        self._tsclient.add_timeseries_values(self._project_id, tds.id, ts1.id, data1)
        data2 = pd.DataFrame([float(7.6)], index=[datetime.datetime.utcnow() + datetime.timedelta(minutes=1)])
        self._tsclient.add_timeseries_values(self._project_id, tds.id, ts1.id, data2)

        # query rainfall
        conditions1 = (
            QueryCondition.create_attribute_query_condition("Item", AttributeOperator.EQUAL, "Rainfall"),
        )
        result1 = [*self._tsclient.query_timeseries(self._project_id, tds.id, conditions1)]

        self.assertIsNotNone(result1)
        self.assertTrue(result1[0].id)
        self._assert_item_definition(item2, result1[0].item)
    
        # query temperature
        conditions2 = [
            QueryCondition.create_attribute_query_condition("Item", AttributeOperator.EQUAL, "Temperature")
        ]
        result2 = [*self._tsclient.query_timeseries(self._project_id, tds.id, conditions2)]
        
        self.assertIsNotNone(result2)
        self.assertTrue(result2[0].id)
        self._assert_item_definition(item1, result2[0].item)

        # query airpressure
        conditions3 = [
            QueryCondition.create_attribute_query_condition("Item", AttributeOperator.EQUAL, "AirPressure")
        ]
        result3 = [*self._tsclient.query_timeseries(self._project_id, tds.id, conditions3)]
        
        self.assertIsNotNone(result3)
        self.assertFalse(result3)

        ids = (result1[0].id, result2[0].id)
        values = [*self._tsclient.get_multiple_timeseries_values(self._project_id, tds.id, ids)]
        
        self.assertEqual(2, len(values))


    def test_add_remove_timeseries_and_timeseries_values_is_ok(self):
        
        dataset = self._tsclient.create_timeseries_dataset(self._project_id, timeseries.TimeSeriesDatasetInput("Py TS with data"))
        
        self.assertIsNotNone(dataset)
        self.assertTrue(dataset.id)

        timeseries_id = f"testTS-{uuid.uuid4()}"

        item = ItemDefinition(
            "Temperature",
            metadata.UnitId.EUMUDEGREECELSIUS,
            metadata.ItemId.EUMITEMPERATURE,
            AttributeDataType.SINGLE
        )

        ts = self._tsclient.add_timeseries_with_id(self._project_id, dataset.id, timeseries_id, item)
        
        self.assertIsNotNone(ts)
        self._assert_item_definition(item, ts.item)

        ts_result = self._tsclient.get_timeseries(self._project_id, dataset.id, ts.id)

        self.assertIsNotNone(ts_result)
        self.assertEqual(ts.id, ts_result.id)
        self._assert_item_definition(ts.item, ts_result.item)

        data = pd.DataFrame([float(12.34)], index=[datetime.datetime.utcnow()])
        self._tsclient.add_timeseries_values(self._project_id, dataset.id, ts.id, data)

        tsv_result = self._tsclient.get_timeseries_values(self._project_id, dataset.id, ts.id)

        self.assertIsNotNone(tsv_result)
        self.assertEqual(data[0][0], tsv_result["values"][0])

        deleted = self._tsclient.delete_timeseries_values(self._project_id, dataset.id, ts.id)

        self.assertTrue(deleted)

        tsv_result2 = self._tsclient.get_timeseries_values(self._project_id, dataset.id, ts.id)
        self.assertIsNotNone(tsv_result2)
        self.assertTrue(tsv_result2.empty)

        deleted_timeseries = self._tsclient.delete_timeseries(self._project_id, dataset.id, ts.id)
        self.assertTrue(deleted_timeseries)

        with self.assertRaises(MikeCloudRestApiException) as context:
            self._tsclient.get_timeseries(self._project_id, dataset.id, ts.id)

    def test_update_timeseries_properties_is_ok(self):
        prop_definition_name = "property1key"
        prop_definition_name2 = "property2key"

        input = timeseries.TimeSeriesDatasetInput(
            name="Py Test TS update properties",
            timeseries_schema = (
                PropertyDefinition(prop_definition_name, PropertyDataType.TEXT),
                PropertyDefinition(prop_definition_name2, PropertyDataType.BOOLEAN)
            )
        )

        dataset = self._tsclient.create_timeseries_dataset(self._project_id, input)

        self.assertIsNotNone(dataset)
        self.assertTrue(dataset.id)

        # add timeseries with properties
        item1 = ItemDefinition(
            "Temperature", 
            metadata.UnitId.EUMUDEGREECELSIUS, 
            metadata.ItemId.EUMITEMPERATURE, 
            AttributeDataType.SINGLE
        )

        properties1 = { prop_definition_name: "property1value"}

        data_fields1 = (
            timeseries.DataFieldDefinition("datafieldname", timeseries.DataFieldDataType.FLAG, flags=(
                timeseries.FlagDefinition(1,"flagdefinitionname", 1947, "flagdefinitiondescription"),
            )),
        )

        ts1 = self._tsclient.add_timeseries(self._project_id, dataset.id, item1, properties1, data_fields1)

        self.assertIsNotNone(ts1)
        self._assert_item_definition(item1, ts1.item)
        self.assertEqual(properties1, ts1.properties)
        self._assert_equal_ts_datafields(data_fields1, ts1.data_fields)

        ts_result = self._tsclient.get_timeseries(self._project_id, dataset.id, ts1.id)
        self.assertIsNotNone(ts_result)
        self.assertEqual(ts1.id, ts_result.id)
        self._assert_item_definition(ts1.item, ts_result.item)
        self.assertEqual(properties1, ts_result.properties)
        self._assert_equal_ts_datafields(data_fields1, ts_result.data_fields)

        # update timeseries property value
        prop_input_value2 = "property1valueupdated"
        prop2_intput_value2 = True
        prop_input = { 
            prop_definition_name: prop_input_value2,
            prop_definition_name2: prop2_intput_value2
        }
        self._tsclient.update_timeseries_properties(self._project_id, dataset.id, ts1.id, prop_input)

        ts_result2 = self._tsclient.get_timeseries(self._project_id, dataset.id, ts1.id)

        self.assertIsNotNone(ts_result2)
        self.assertEqual(ts1.id, ts_result2.id)
        self._assert_item_definition(ts1.item, ts_result2.item)
        self.assertEqual(prop_input, ts_result2.properties)
        self._assert_equal_ts_datafields(data_fields1, ts_result2.data_fields)


    def test_update_timeseries_property_is_ok(self):
        prop_definition_name = "property1key"
        input = timeseries.TimeSeriesDatasetInput(
            name = "Py Test TS update property",
            timeseries_schema=[PropertyDefinition(prop_definition_name, PropertyDataType.TEXT)]
        )

        dataset = self._tsclient.create_timeseries_dataset(self._project_id, input)

        self.assertIsNotNone(dataset)
        self.assertTrue(dataset.id)

        item1 = ItemDefinition(
            "Temperature",
            metadata.UnitId.EUMUDEGREECELSIUS,
            metadata.ItemId.EUMITEMPERATURE,
            AttributeDataType.SINGLE
        )

        properties1 = { prop_definition_name: "property1value" }

        data_fields1 = (
            timeseries.DataFieldDefinition(
                "datafieldname",
                timeseries.DataFieldDataType.FLAG,
                flags=(timeseries.FlagDefinition(1, "flagdefinitionname", 1947, "flagdefinitiondescription"),)
                ),
        )

        ts1 = self._tsclient.add_timeseries(self._project_id, dataset.id, item1, properties1, data_fields1)

        self.assertIsNotNone(ts1)
        self.assertTrue(ts1.id)
        self._assert_item_definition(item1, ts1.item)
        self.assertEqual(properties1, ts1.properties)
        self._assert_equal_ts_datafields(data_fields1, ts1.data_fields)

        ts_result = self._tsclient.get_timeseries(self._project_id, dataset.id, ts1.id)
        self.assertIsNotNone(ts_result)
        self.assertEqual(ts1.id, ts_result.id)
        self._assert_item_definition(ts1.item, ts_result.item)
        self.assertEqual(properties1, ts_result.properties)
        self._assert_equal_ts_datafields(data_fields1, ts_result.data_fields)

        # update timeseries property value
        prop_input_value2 = "property1valueupdated"
        self._tsclient.update_timeseries_property(self._project_id, dataset.id, ts1.id, prop_definition_name, prop_input_value2)

        ts_result2 = self._tsclient.get_timeseries(self._project_id, dataset.id, ts1.id)
        self.assertIsNotNone(ts_result2)
        self.assertEqual(ts1.id, ts_result2.id)
        self._assert_item_definition(ts1.item, ts_result2.item)
        properties1.update({prop_definition_name: prop_input_value2})
        self.assertEqual(properties1, ts_result2.properties)
        self._assert_equal_ts_datafields(data_fields1, ts_result2.data_fields)

    def test_remove_timeseries_and_timeseries_values_with_interval_is_ok(self):
        dataset = self._tsclient.create_timeseries_dataset(self._project_id, timeseries.TimeSeriesDatasetInput("Py Test TS With Data"))

        time.sleep(2) # let events propagate in local development

        self.assertIsNotNone(dataset)
        self.assertTrue(dataset.id)

        item = ItemDefinition(
            "Temperature",
            metadata.UnitId.EUMUDEGREECELSIUS,
            metadata.ItemId.EUMITEMPERATURE,
            AttributeDataType.SINGLE
        )

        ts = self._tsclient.add_timeseries(self._project_id, dataset.id, item)
        
        self.assertIsNotNone(ts)
        self.assertTrue(ts.id)
        self._assert_item_definition(item, ts.item)

        start_time = datetime.datetime.utcnow()
        dates = []
        values = []
        for i in range(0, 230):
            dates.append(start_time + datetime.timedelta(minutes=i))
            values.append(random.uniform(0.0, 1000.0))
        data = pd.DataFrame(values, index=dates)

        self._tsclient.add_timeseries_values(self._project_id, dataset.id, ts.id, data)

        from_ = start_time + datetime.timedelta(minutes=15)
        to = start_time + datetime.timedelta(minutes=30)
        tsv_result = self._tsclient.get_timeseries_values(self._project_id, dataset.id, ts.id, from_, to)

        self.assertIsNotNone(tsv_result)
        self.assertEqual(tsv_result.shape, (16, 1))
        self.assertTrue(tsv_result["values"].values.tolist(), values)
        self.assertTrue(tsv_result.index.tolist(), dates)

        deleted = self._tsclient.delete_timeseries_values(self._project_id, dataset.id, ts.id, from_, to)

        self.assertTrue(deleted)

        tsv_result2 = self._tsclient.get_timeseries_values(self._project_id, dataset.id, ts.id, from_, to)
        self.assertIsNotNone(tsv_result2)
        self.assertTrue(tsv_result2.empty)


if __name__ == "__main__":
    unittest.main()


#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.timeseriesgen import TimeSeriesGenClientV2
from dhi.platform.fmt import Format

def main():
    args = ClientArgs.ParseForDatasetPos(description="List time series")
    ClientConfig.UpdateDatasetFromConfiguration(args)
    clientv2 = TimeSeriesGenClientV2(**vars(args))

    response = clientv2.GetAllTimeSeriesV2(args.projectid, args.datasetid)

    tablefmt = "{!s:32}\t{}"
    tablefields = ["id", "item"]
    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.metadatahelper import MetadataClientV3Contracts
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("-f", "--inputfile", help="Input file path, '-' to read stdin")
    parser.add_argument("--name", help="Project name")
    parser.add_argument("--description", help="Project description")
    parser.add_argument("--accesslevel", choices=["Confidential", "Private", "Shared"], help="Project accel level")
    parser.add_argument("--metadata", help="Project metadata JSON, {}")
    parser.add_argument("--settings", help="Project settings JSON, {}")
    parser.add_argument("--members", help="Project members array JSON, []")

def main():
    args = ClientArgs.ParsePlatform(description="Create project", init=__initParser, defaultformat="yaml")
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    input = MetadataClientV3Contracts.PrepareCreateProjectInput(
        ClientArgs.LoadJson(args.inputfile),
        name=args.name,
        description=args.description,
        accesslevel=args.accesslevel,
        metadata=ClientArgs.LoadJsonStr(args.metadata),
        settings=ClientArgs.LoadJsonStr(args.settings),
        members=ClientArgs.LoadJsonStr(args.members))
    response = clientv3.CreateProjectV3(input)

    tablefields = ["id", "name"]
    Format.FormatResponse(response, lambda r: r.Body, args.format, None, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.rawgen import RawGenClientV2
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("--count", help="Count of staging blobs to get")

def main():
    args = ClientArgs.ParseForProject(description="Get staging urls", init=__initParser, defaultformat="yaml")
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv2 = RawGenClientV2(**vars(args))

    response = clientv2.GetStagingUrls(args.projectid, args.count)

    tablefmt = "{}"
    tablefields = ["url"]
    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def initParser(parser):
    parser.add_argument("-l", "--limit", default=50, help="Limit output size in one call")

def main():
    args = ClientArgs.ParsePlatform(description="List customers", init=initParser)
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    response = clientv3.GetUserList()

    tablefmt = "{!s:32}\t{!s:13}\t{!s:23}\t{!s:7}\t{!s:32}\t{!s:13}"
    tablefields = ["userId", "displayName", "email", "isAdmin", "customerId", "customerName"]
    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.timeserieshelper import TimeSeriesV2Contracts
from dhi.platform.generated.timeseriesgen import TimeSeriesGenClientV2
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("-f", "--inputfile", help="Input file path, '-' to read stdin")
    parser.add_argument("--id", help="Time series id")
    parser.add_argument("--name", help="Time series name")
    parser.add_argument("--unit", help="Time series unit")
    parser.add_argument("--item", help="Time series item (eum)")
    parser.add_argument("--datatype", help="Time series data type")
    parser.add_argument("--timeseriestype", help="Time series type")
    parser.add_argument("--properties", help="Time series settings JSON, {}")
    parser.add_argument("--datafileds", help="Time series data fields JSON, []")

def main():
    args = ClientArgs.ParseForDatasetPos(description="Add time series", init=__initParser, defaultformat="yaml")
    ClientConfig.UpdateDatasetFromConfiguration(args)
    clientv2 = TimeSeriesGenClientV2(**vars(args))

    input = TimeSeriesV2Contracts.PrepareAddTimeSeriesV2Input(
        ClientArgs.LoadJson(args.inputfile),
        id=args.id,
        name=args.name,
        unit=args.unit,
        item=args.item,
        datatype=args.datatype,
        timeseriestype=args.timeseriestype,
        properties=ClientArgs.LoadJsonStr(args.properties),
        datafileds=ClientArgs.LoadJsonStr(args.datafileds))
    response = clientv2.AddTimeSeriesV2(args.projectid, input, args.datasetid)

    tablefields = ["id", "name"]
    Format.FormatResponse(response, lambda r: r.Body, args.format, None, tablefields)

if __name__ == '__main__':
    main()



# Copyright (c) 2021 DHI A/S - DHI Water Environment Health 
# All rights reserved.
# 
# This code is licensed under the MIT License.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import datetime
from typing import List

from dhi.platform.base.exceptions import MikeCloudException

from dhi.platform.base.utils import sanitize_from_to
from dhi.platform.eventing import EngineSubscription
from .generated.enginegen import EngineExecutionDiagnosticsOutputV2, EngineExecutionInputOutputV2, EngineExecutionInputParametersV2, EngineExecutionOutputV2, EngineExecutionParametersV2, EngineExecutionSummaryOutputV2, EngineGenClientV2, EngineGetOutputV2, EngineInputItemV2, EngineModelItemV2, EngineOutputItemV2, EngineOutputPlatformItemV2, EngineRunOutputV2, EngineRunParameterV2, GetConfigurationOutputV2, OptionsV2

class EngineExecutionInputBuilderBase():
    
    def __init__(self) -> None:
        self._scenario_name = None
        self._options = None
        self._output = None
        self._input = None

    @property 
    def scenario_name(self):
        return self._scenario_name

    @property
    def options(self):  
        return self._options
    
    @property
    def output(self):
        return self._output
    
    @property
    def input(self):
        return self._input
    
    def _validate(self):
        messages = []
        
        if self._options is None:
            messages.append("Options are required")
        else:
            opts = self._options
            if not opts.poolType:
                messages.append("Pool type option is required")
            if not opts.nodeCount:
                messages.append("Node count option is required")
            if not opts.maxExecutionElapsedTimeHours:
                messages.append("Positive Maximum Execution Elaps Time Hous option is required")
        
        if messages:
            raise MikeCloudException("Execution input is invalid", messages)
    
    def with_options(self, pool_type:str, node_count:str, max_elapsed_time:float=24.0):
        max_elapsed_time = float(max_elapsed_time)
        self._options = OptionsV2(pool_type, node_count, max_elapsed_time)
        return self
    
    def with_max_execution_elapsed_time_hours(self, hours:int):
        self._maxExecutionElapsedTimeHours = hours
        return self
    
    def with_scenario_name(self, name:str):
        self._scenario_name = name
        return self
    
    def with_output(self, output:List[EngineOutputItemV2]):
        self._output = output
        return self


class EngineExecutionInputBuilder(EngineExecutionInputBuilderBase):

    def __init__(self) -> None:
        super().__init__()
        self._input = None
        self._platform_output = None

    def _validate(self):
        super()._validate()
    
    def with_input(self, input:List[EngineInputItemV2]):
        self._input = input
        return self

    def with_platform_output(self, platform_output:List[EngineOutputPlatformItemV2]):
        self._platform_output = platform_output
        return self

    def build(self) -> EngineExecutionParametersV2:
        
        self._validate()

        parameters = EngineExecutionParametersV2(
            self._input,
            self.output,
            self._platform_output,
            self.options,
            self.scenario_name
        )

        return parameters


class EngineExecutionInputParametersBuilder(EngineExecutionInputBuilderBase):
    
    def __init__(self) -> None:
        super().__init__()
        self._models = None
    
    def _validate(self):
        super()._validate()
        messages = []

        for model in self._models:
            if not model.resultsRelativePath:
                messages.append(f"Model {model} must have 'resultsRelativePath'")

        if messages:
            raise MikeCloudException('Cannot build EngineExecutionParametersV2', messages)

    def with_models(self, models=List[EngineModelItemV2]):
        self._models = models
        return self

    def build(self) -> EngineExecutionParametersV2:
        
        self._validate()

        parameters = EngineExecutionInputParametersV2(
            self._models,
            self.output,
            self.options,
            self.scenario_name
        )

        return parameters


class EngineClient(EngineGenClientV2):
    def __init__(self, inspectFnc=EngineGenClientV2.DefaultInspectFnc, **kwargs):
        super().__init__(inspectFnc, **kwargs)

    def _page_to_list(cls, response):
        return [cls.from_dict(d) for d in response.Body["data"]]
    
    def get_execution(self, project_id, execution_id) -> EngineExecutionOutputV2:
        """
        Get multidimensional dataset details
        
        :param project_id: ID of the project to get an execution from
        :param execution_id: ID of the execution
        :return: engine execution details
        :rtype: EngineExecutionOutputV2
        """
        response = super().GetExecution(project_id, execution_id)
        return EngineExecutionOutputV2.from_dict(response.Body)

    def list_executions(self, project_id, from_:datetime.datetime=None, to:datetime.datetime=None) -> List[EngineExecutionSummaryOutputV2]:
        """
        List engine executions
        
        :param project_id: ID of the project to list executions from
        :param from_: Executions started after this time
        :param to_: Executions started before this time
        :return: list of engine execution details
        :rtype: List[EngineExecutionSummaryOutputV2]
        """
        from_, to = sanitize_from_to(from_, to)
        data = []
        cursor = None
        response = super().GetExecutions(project_id, starttime=from_, endtime=to, cursor=cursor)
        cursor = response.Body["cursor"]
        data += self._page_to_list(EngineExecutionSummaryOutputV2, response)
        while cursor:
            response = super().GetExecutions(project_id, starttime=from_, endtime=to, cursor=cursor)
            data += self._page_to_list(EngineExecutionSummaryOutputV2, response)
            cursor = response.Body["cursor"]
        return data

    def cancel_execution(self, project_id, execution_id) -> bool:
        """
        Cancel engine execution
        
        :param project_id: ID of the project cancel execution in
        :param execution_id: ID of the execution
        :return: true if the cancellation request succeeded, otherwise false.
        :rtype: bool
        """
        response = super().CancelExecution(project_id, execution_id)
        return response.IsOk

    def delete_execution(self, project_id, execution_id) -> bool:
        """
        Delete engine execution
        
        :param project_id: ID of the project to delete execution in
        :param execution_id: ID of the execution
        :return: true if the deletion request succeeded, otherwise false.
        :rtype: bool
        """
        response = super().DeleteExecution(project_id, execution_id)
        return response.IsOk

    def list_configurations(self, project_id) -> List[GetConfigurationOutputV2]:
        """
        List configuration, i.e. information on available machine types

        :param project_id: ID of the project for the project context
        :return: List[GetConfigurationOutputV2]
        :rtype: List of available configurations
        """
        response = super().GetAllConfigurations(project_id)
        return self._page_to_list(GetConfigurationOutputV2, response)

    def list_engines(self, project_id) -> List[EngineGetOutputV2]:
        """
        List available engines

        :param project_id: ID of the project for the project context
        :return: List[EngineGetOutputV2]
        :rtype: List of available engines
        """
        response = super().GetAllEngines(project_id)
        return self._page_to_list(EngineGetOutputV2, response)
        
    def get_execution_diagnostics(self, project_id, execution_id) -> EngineExecutionDiagnosticsOutputV2:
        """
        Delete engine execution
        
        :param project_id: ID of the project to delete execution in
        :param execution_id: ID of the execution
        :return: true if the deletion request succeeded, otherwise false.
        :rtype: bool
        """
        response = super().GetExecutionDiagnostics(project_id, execution_id)
        return EngineExecutionDiagnosticsOutputV2.from_dict(response.Body)

    def get_execution_inputs(self, project_id, execution_id) -> EngineExecutionInputOutputV2:
        """
        Get execution inputs
        
        :param project_id: ID of the project containing the execution
        :param execution_id: ID of the execution
        :return: Information about what data were used as input for a specific execution
        :rtype: EngineExecutionInputOutputV2
        """
        response = super().GetExecutionInputs(project_id, execution_id)
        return EngineExecutionInputOutputV2.from_dict(response.Body)

    
    def run_execution(self, project_id, input:EngineExecutionParametersV2, engine_message_handler:EngineSubscription) -> EngineRunOutputV2:
        """
        Run a new execution
        
        :param project_id: ID of the project for the project context
        :param input: input for the execution
        :return: Details about the executed execution
        :rtype: EngineRunOutputV2
        """
        response = super().RunExecution(project_id, input.to_dict())
        engine_run_output = EngineRunOutputV2.from_dict(response.Body)
        execution_id = engine_run_output.executionId
        if engine_message_handler:
            engine_message_handler.set_resource_id(execution_id)
        return engine_run_output
        
    
    def run_execution_with_platform_data(self, project_id, input:EngineExecutionInputParametersV2, engine_message_handler:EngineSubscription) -> EngineRunOutputV2:
        """
        Run a new execution with inputs stored in MIKE Cloud Platform
        
        :param project_id: ID of the project for the project context
        :param input: input for the execution
        :return: Details about the executed execution
        :rtype: EngineRunOutputV2
        """

        response = super().RunExecutionWithPlatformData(project_id, input.to_dict())
        engine_run_output = EngineRunOutputV2.from_dict(response.Body)
        execution_id = engine_run_output.executionId
        if engine_message_handler:
            engine_message_handler.set_resource_id(execution_id)
        return engine_run_output
    



if __name__ == '__main__':
    print(__file__)
    print(dir())



# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
# https://developers.google.com/protocol-buffers/
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
# copyright notice, this list of conditions and the following disclaimer
# in the documentation and/or other materials provided with the
# distribution.
#     * Neither the name of Google Inc. nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Constants and static functions to support protocol buffer wire format."""

__author__ = 'robinson@google.com (Will Robinson)'

import struct
#from google.protobuf import descriptor
#from google.protobuf import message
from . import descriptor
from . import message


TAG_TYPE_BITS = 3  # Number of bits used to hold type info in a proto tag.
TAG_TYPE_MASK = (1 << TAG_TYPE_BITS) - 1  # 0x7

# These numbers identify the wire type of a protocol buffer value.
# We use the least-significant TAG_TYPE_BITS bits of the varint-encoded
# tag-and-type to store one of these WIRETYPE_* constants.
# These values must match WireType enum in google/protobuf/wire_format.h.
WIRETYPE_VARINT = 0
WIRETYPE_FIXED64 = 1
WIRETYPE_LENGTH_DELIMITED = 2
WIRETYPE_START_GROUP = 3
WIRETYPE_END_GROUP = 4
WIRETYPE_FIXED32 = 5
_WIRETYPE_MAX = 5


# Bounds for various integer types.
INT32_MAX = int((1 << 31) - 1)
INT32_MIN = int(-(1 << 31))
UINT32_MAX = (1 << 32) - 1

INT64_MAX = (1 << 63) - 1
INT64_MIN = -(1 << 63)
UINT64_MAX = (1 << 64) - 1

# "struct" format strings that will encode/decode the specified formats.
FORMAT_UINT32_LITTLE_ENDIAN = '<I'
FORMAT_UINT64_LITTLE_ENDIAN = '<Q'
FORMAT_FLOAT_LITTLE_ENDIAN = '<f'
FORMAT_DOUBLE_LITTLE_ENDIAN = '<d'


# We'll have to provide alternate implementations of AppendLittleEndian*() on
# any architectures where these checks fail.
if struct.calcsize(FORMAT_UINT32_LITTLE_ENDIAN) != 4:
  raise AssertionError('Format "I" is not a 32-bit number.')
if struct.calcsize(FORMAT_UINT64_LITTLE_ENDIAN) != 8:
  raise AssertionError('Format "Q" is not a 64-bit number.')


def PackTag(field_number, wire_type):
  """Returns an unsigned 32-bit integer that encodes the field number and
  wire type information in standard protocol message wire format.

  Args:
    field_number: Expected to be an integer in the range [1, 1 << 29)
    wire_type: One of the WIRETYPE_* constants.
  """
  if not 0 <= wire_type <= _WIRETYPE_MAX:
    raise message.EncodeError('Unknown wire type: %d' % wire_type)
  return (field_number << TAG_TYPE_BITS) | wire_type


def UnpackTag(tag):
  """The inverse of PackTag().  Given an unsigned 32-bit number,
  returns a (field_number, wire_type) tuple.
  """
  return (tag >> TAG_TYPE_BITS), (tag & TAG_TYPE_MASK)


def ZigZagEncode(value):
  """ZigZag Transform:  Encodes signed integers so that they can be
  effectively used with varint encoding.  See wire_format.h for
  more details.
  """
  if value >= 0:
    return value << 1
  return (value << 1) ^ (~0)


def ZigZagDecode(value):
  """Inverse of ZigZagEncode()."""
  if not value & 0x1:
    return value >> 1
  return (value >> 1) ^ (~0)



# The *ByteSize() functions below return the number of bytes required to
# serialize "field number + type" information and then serialize the value.


def Int32ByteSize(field_number, int32):
  return Int64ByteSize(field_number, int32)


def Int32ByteSizeNoTag(int32):
  return _VarUInt64ByteSizeNoTag(0xffffffffffffffff & int32)


def Int64ByteSize(field_number, int64):
  # Have to convert to uint before calling UInt64ByteSize().
  return UInt64ByteSize(field_number, 0xffffffffffffffff & int64)


def UInt32ByteSize(field_number, uint32):
  return UInt64ByteSize(field_number, uint32)


def UInt64ByteSize(field_number, uint64):
  return TagByteSize(field_number) + _VarUInt64ByteSizeNoTag(uint64)


def SInt32ByteSize(field_number, int32):
  return UInt32ByteSize(field_number, ZigZagEncode(int32))


def SInt64ByteSize(field_number, int64):
  return UInt64ByteSize(field_number, ZigZagEncode(int64))


def Fixed32ByteSize(field_number, fixed32):
  return TagByteSize(field_number) + 4


def Fixed64ByteSize(field_number, fixed64):
  return TagByteSize(field_number) + 8


def SFixed32ByteSize(field_number, sfixed32):
  return TagByteSize(field_number) + 4


def SFixed64ByteSize(field_number, sfixed64):
  return TagByteSize(field_number) + 8


def FloatByteSize(field_number, flt):
  return TagByteSize(field_number) + 4


def DoubleByteSize(field_number, double):
  return TagByteSize(field_number) + 8


def BoolByteSize(field_number, b):
  return TagByteSize(field_number) + 1


def EnumByteSize(field_number, enum):
  return UInt32ByteSize(field_number, enum)


def StringByteSize(field_number, string):
  return BytesByteSize(field_number, string.encode('utf-8'))


def BytesByteSize(field_number, b):
  return (TagByteSize(field_number)
          + _VarUInt64ByteSizeNoTag(len(b))
          + len(b))


def GroupByteSize(field_number, message):
  return (2 * TagByteSize(field_number)  # START and END group.
          + message.ByteSize())


def MessageByteSize(field_number, message):
  return (TagByteSize(field_number)
          + _VarUInt64ByteSizeNoTag(message.ByteSize())
          + message.ByteSize())


def MessageSetItemByteSize(field_number, msg):
  # First compute the sizes of the tags.
  # There are 2 tags for the beginning and ending of the repeated group, that
  # is field number 1, one with field number 2 (type_id) and one with field
  # number 3 (message).
  total_size = (2 * TagByteSize(1) + TagByteSize(2) + TagByteSize(3))

  # Add the number of bytes for type_id.
  total_size += _VarUInt64ByteSizeNoTag(field_number)

  message_size = msg.ByteSize()

  # The number of bytes for encoding the length of the message.
  total_size += _VarUInt64ByteSizeNoTag(message_size)

  # The size of the message.
  total_size += message_size
  return total_size


def TagByteSize(field_number):
  """Returns the bytes required to serialize a tag with this field number."""
  # Just pass in type 0, since the type won't affect the tag+type size.
  return _VarUInt64ByteSizeNoTag(PackTag(field_number, 0))


# Private helper function for the *ByteSize() functions above.

def _VarUInt64ByteSizeNoTag(uint64):
  """Returns the number of bytes required to serialize a single varint
  using boundary value comparisons. (unrolled loop optimization -WPierce)
  uint64 must be unsigned.
  """
  if uint64 <= 0x7f: return 1
  if uint64 <= 0x3fff: return 2
  if uint64 <= 0x1fffff: return 3
  if uint64 <= 0xfffffff: return 4
  if uint64 <= 0x7ffffffff: return 5
  if uint64 <= 0x3ffffffffff: return 6
  if uint64 <= 0x1ffffffffffff: return 7
  if uint64 <= 0xffffffffffffff: return 8
  if uint64 <= 0x7fffffffffffffff: return 9
  if uint64 > UINT64_MAX:
    raise message.EncodeError('Value out of range: %d' % uint64)
  return 10


NON_PACKABLE_TYPES = (
  descriptor.FieldDescriptor.TYPE_STRING,
  descriptor.FieldDescriptor.TYPE_GROUP,
  descriptor.FieldDescriptor.TYPE_MESSAGE,
  descriptor.FieldDescriptor.TYPE_BYTES
)


def IsTypePackable(field_type):
  """Return true iff packable = true is valid for fields of this type.

  Args:
    field_type: a FieldDescriptor::Type value.

  Returns:
    True iff fields of this type are packable.
  """
  return field_type not in NON_PACKABLE_TYPES


#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV2
from dhi.platform.metadatahelper import MetadataClientV2Helper
from dhi.platform.fmt import Format

def initParser(parser):
    parser.add_argument("transferid", help="Transfer id")
    parser.add_argument("--outputpath", help="Output path")

def main():
    args = ClientArgs.ParsePlatform(description="Transfer download", init=initParser, defaultformat="yaml")
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv2 = MetadataGenClientV2(**vars(args))

    response = clientv2.GetTransferV2(args.transferid)

    Format.FormatResponse(response, lambda r: r.Body, args.format)

    if response.Body.get("status") == "Completed":
        url = response.Body.get("downloadPath")
        if url:
            MetadataClientV2Helper.DownloadBlobToFile(url, args.outputpath)

if __name__ == '__main__':
    main()



# Copyright (c) 2021 DHI A/S - DHI Water Environment Health 
# All rights reserved.
# 
# This code is licensed under the MIT License.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import json, pathlib
from datetime import datetime
from types import SimpleNamespace
from typing import Callable


class ClientConfig:
    __cachedconfigname = None
    __cachedcfg = None
    __cacheddefaultenv = None
    __cacheddefaultenvname = None

    @classmethod
    def GetEnvironmentFromConfiguration(cls, environment, configname=None):
        if not environment:
            environment = cls.GetDefaultEnvironmentName(environment, configname)
        return (environment, cls.GetEnvironment(environment, configname))

    @staticmethod
    def GetConfigname(args, configname=None):
        return configname if configname else args.config if hasattr(args, "config") else None

    @classmethod
    def UpdateEnvironmentFromConfiguration(cls, args, configname=None):
        configname = cls.GetConfigname(args, configname)
        (args.environment, envObj) = cls.GetEnvironmentFromConfiguration(args.environment, configname)
        return envObj

    @classmethod
    def UpdatePlatformFromConfiguration(cls, args, configname=None):
        envObj = cls.UpdateEnvironmentFromConfiguration(args, configname)
        if envObj:
            if not args.apikey:
                args.apikey = cls.GetApiKey(envObj)
            if args.preferapikey == None:
                args.preferapikey = cls.GetPreferApiKey(envObj)
            if not (args.preferapikey and args.apikey):
                if tokeninfo := cls.GetAccessTokenInfo(envObj):
                    if cls.IsValid(tokeninfo):
                        args.accesstoken = cls.GetToken(tokeninfo, args.environment)
                    elif cls.IsExpired(tokeninfo):
                        args.accesstoken = cls.RefreshToken(tokeninfo, args.environment)
        return envObj

    @classmethod
    def UpdateProjectFromConfiguration(cls, args, configname=None):
        envObj = cls.UpdatePlatformFromConfiguration(args, configname)
        if envObj:
            if not hasattr(args, 'projectid') or not args.projectid:
                args.projectid = cls.GetProjectid(envObj)
        return envObj

    @classmethod
    def UpdateDatasetFromConfiguration(cls, args, configname=None):
        envObj = cls.UpdateProjectFromConfiguration(args, configname)
        if envObj:
            if not hasattr(args, 'datasetid') or not args.projectid:
                args.datasetid = cls.GetDatasetid(envObj)
        return envObj

    @staticmethod
    def GetConfigurationFilePath(configname=None):
        filename = f".dhiplatform-{configname}.json" if configname else ".dhiplatform.json"
        return pathlib.Path.home().joinpath(filename)

    @classmethod
    def GetConfiguraton(cls, configname=None):
        if cls.__cachedcfg == None or cls.__cachedconfigname != configname:
            cfgfilepath = cls.GetConfigurationFilePath(configname)
            cls.__cachedconfigname = configname
            cls.__cachedcfg = cls.__LoadFile(cfgfilepath)
        return cls.__cachedcfg

    @classmethod
    def GetDefaultEnvironment(cls, environment=None, configname=None):
        if cls.__cacheddefaultenv == None:
            def getdefaultenvironment(cfgObj, envObj):
                if envObj != None:
                    return SimpleNamespace(envobj=envObj, envname=environment)
                defaultenv = cfgObj.get("default")
                return SimpleNamespace(envobj=cfgObj.get(defaultenv), envname=defaultenv) if defaultenv else None
            tmp = cls.__LoadConfiguration(environment, getdefaultenvironment, configname)
            if tmp:
                cls.__cacheddefaultenv = tmp.envobj
                cls.__cacheddefaultenvname = tmp.envname
        return cls.__cacheddefaultenv

    @classmethod
    def GetDefaultEnvironmentName(cls, environment=None, configname=None):
        cls.GetDefaultEnvironment(environment, configname)
        return cls.__cacheddefaultenvname

    @classmethod
    def GetDefaultEnvironmentAccessToken(cls, environment=None, configname=None):
        envObj = cls.GetDefaultEnvironment(environment, configname)
        return envObj.get("accesstoken") if envObj else None

    @classmethod
    def GetEnvironment(cls, environment, configname=None):
        return cls.__LoadConfiguration(environment, lambda _, envObj: envObj, configname)

    @classmethod
    def GetToken(cls, tokeninfo, env):
        result = tokeninfo.get("accesstoken") if tokeninfo else None
        if not result:
            result = cls.RefreshToken(tokeninfo, env)
        return result

    @classmethod
    def RefreshToken(cls, tokeninfo, env):
        from dhi.platform.authentication import ClientAuthentication
        (tokeninfo, result) = ClientAuthentication.RefreshToken(env, tokeninfo)
        if tokeninfo:
            cls.SaveUserTokenInfo(env, tokeninfo)
        return result

    @classmethod
    def AcquireAccessToken(cls, environment, configname=None, forcelogin=True):
        if not forcelogin:
            (environment, envObj) = cls.GetEnvironmentFromConfiguration(environment, configname)
            if tokeninfo := cls.GetAccessTokenInfo(envObj):
                if cls.IsValid(tokeninfo):
                    return cls.GetToken(tokeninfo, environment)
                elif cls.IsExpired(tokeninfo):
                    return cls.RefreshToken(tokeninfo, environment)
        from dhi.platform.authentication import ClientAuthentication
        (tokeninfo, result) = ClientAuthentication.AcquireTokenInteractively(environment)
        if tokeninfo:
            cls.SaveUserTokenInfo(environment, tokeninfo)
        return result

    @staticmethod
    def GetUsername(tokeninfo):
        return tokeninfo.get("username") if tokeninfo else None

    @staticmethod
    def GetExpiresOn(tokeninfo):
        tmp = tokeninfo.get("expireson") if tokeninfo else None
        return datetime.fromtimestamp(tmp) if tmp else None

    @staticmethod
    def IsExpired(tokeninfo):
        tmp = tokeninfo.get("expireson") if tokeninfo else None
        return datetime.fromtimestamp(tmp) < datetime.now() if tmp else None

    @staticmethod
    def IsValid(tokeninfo):
        tmp = tokeninfo.get("expireson") if tokeninfo else None
        return datetime.fromtimestamp(tmp) >= datetime.now() if tmp else False

    @staticmethod
    def GetAccessTokenInfo(envObj):
        return envObj.get("accesstoken") if envObj else None

    @staticmethod
    def GetApiKey(envObj):
        return envObj.get("apikey") if envObj else None

    @staticmethod
    def GetPreferApiKey(envObj):
        return envObj.get("preferapikey") if envObj else None

    @staticmethod
    def GetProjectid(envObj):
        return envObj.get("projectid") if envObj else None

    @staticmethod
    def GetDatasetid(envObj):
        return envObj.get("datasetid") if envObj else None

    @classmethod
    def ClearInfo(cls, environment, clearlogin=False, clearapikey=False, configname=None):
        def clearinfo(cfgObj, envObj):
            if clearlogin:
                envObj.pop("accesstoken", None)
            if clearapikey:
                envObj.pop("apikey", None)
                envObj.pop("preferapikey", None)
            cfgObj["default"] = environment
        cls.__UpdateConfiguration(environment, clearinfo, configname)

    @classmethod
    def SaveUserTokenInfo(cls, environment, token, configname=None):
        def settoken(cfgObj, envObj):
            envObj["accesstoken"] = token
            if token:
                envObj["preferapikey"] = False
            cfgObj["default"] = environment
        cls.__UpdateConfiguration(environment, settoken, configname)

    @classmethod
    def SaveApiKey(cls, environment, apikey, configname=None):
        def setapikey(cfgObj, envObj):
            envObj["apikey"] = apikey
            envObj["preferapikey"] = apikey != None
            cfgObj["default"] = environment
        cls.__UpdateConfiguration(environment, setapikey, configname)

    @classmethod
    def SetPreferApiKey(cls, environment, value, configname=None):
        def setpreferapikey(cfgObj, envObj):
            envObj["preferapikey"] = value
            cfgObj["default"] = environment
        cls.__UpdateConfiguration(environment, setpreferapikey, configname)

    @staticmethod
    def __LoadFile(filepath, default={}):
        obj = None
        if filepath.exists():
            with open(filepath, "r") as f:
                try:
                    obj = json.load(f)
                except:
                    obj = None
        return obj if obj else default

    @staticmethod
    def __SaveFile(obj, filepath):
        with open(filepath, "w") as f:
            json.dump(obj, f, indent=True)

    @staticmethod
    def __GetEnvironment(obj, environment):
        if environment:
            envObj = obj.get(environment)
            if not envObj:
                envObj = {}
                obj[environment] = envObj
            return envObj
        return None

    @classmethod
    def __LoadConfiguration(cls, environment, getter: Callable, configname=None):
        cfgObj = cls.GetConfiguraton(configname)
        if getter:
            envObj = cls.__GetEnvironment(cfgObj, environment)
            return getter(cfgObj, envObj)
        return None

    @classmethod
    def __UpdateConfiguration(cls, environment, update: Callable, configname=None):
        cfgObj = cls.GetConfiguraton(configname)
        if update:
            cfgfilepath = cls.GetConfigurationFilePath(configname)
            cfgObj = cls.__LoadFile(cfgfilepath)
            envObj = cls.__GetEnvironment(cfgObj, environment)
            update(cfgObj, envObj)
            cls.__SaveFile(cfgObj, cfgfilepath)


if __name__ == '__main__':
    print(__file__)
    print(dir())



import unittest
import datetime
from unittest.case import _AssertRaisesContext
import uuid
from dhi.platform import metadata
from dhi.platform.base.exceptions import MikeCloudRestApiException
from .testcredentials import TEST_IDENTITY

class TestCreateProject(unittest.TestCase):

    _verbosity = 0
    
    def setUp(self) -> None:
        self._metadataclient = metadata.MetadataClient(verbose=self._verbosity, identity=TEST_IDENTITY)
        self._projects_to_delete = []

    def tearDown(self) -> None:
        super().tearDown()
        for p in self._projects_to_delete:
            try:
                self._metadataclient.delete_project(p, permanently=True)
            except:
                pass
        self._projects_to_delete = []

    def _create_project(self, title_root="Python test ", description="Project created by Python SDK test"):
        stamp = datetime.datetime.now().strftime('%Y%m%d%H%M%S')
        name = title_root + stamp
        projectInput = metadata.CreateProjectInput(name, description)
        project = self._metadataclient.create_project(projectInput)
        return project

    def test_metadata_client_can_initialize_is_ok(self):
        metadataclient = metadata.MetadataClient(verbose=self._verbosity, identity=TEST_IDENTITY)
        self.assertIsNotNone(metadataclient)
    
    def test_create_get_and_destroy_project_is_ok(self):
        stamp = datetime.datetime.now().strftime('%Y%m%d%H%M%S')
        name = 'Python test ' + stamp
        projectInput = metadata.CreateProjectInput(name, 'Project created by Python SDK test')
        project = self._metadataclient.create_project(projectInput)
        self.assertIsNotNone(project)
        self.assertEqual(project.name, name)
        self._projects_to_delete.append(project.id)
        single_project = self._metadataclient.get_project(project.id)
        self.assertIsNotNone(single_project)
        self.assertEqual(project.id, single_project.id)
        self.assertEqual(project.name, single_project.name)
        self._metadataclient.delete_project(project.id, permanently=True)
    
    def test_get_nonexisting_project_raises(self):
        project_id = str(uuid.uuid4())
        with self.assertRaises(MikeCloudRestApiException):
            self._metadataclient.get_project(project_id)
    
    def test_list_projects_is_ok(self):
        projects = self._metadataclient.list_projects()
        self.assertTrue(projects)
        first_project = next(projects)
        self.assertIsNotNone(first_project)
        self.assertTrue(first_project.id)
        self.assertTrue(first_project.name)

    def test_update_project_is_ok(self):
        stamp = datetime.datetime.now().strftime('%Y%m%d%H%M%S')
        name = 'Python test ' + stamp
        description = 'Project created by Python SDK test'
        projectInput = metadata.CreateProjectInput(name, description)
        project = self._metadataclient.create_project(projectInput)
        self.assertIsNotNone(project)
        self._projects_to_delete.append(project.id)

        update_project_input = metadata.UpdateProjectInput(project.id, name + " Updated", description + " Updated", metadata={"foo": "bar", "spam": 1})
        updated = self._metadataclient.update_project(update_project_input)
        self.assertEqual(updated.name, name + " Updated")
        self.assertEqual(updated.description, description + " Updated")
        self.assertTrue("foo" in updated.metadata)
        self.assertTrue("spam" in updated.metadata)
        self.assertEqual(updated.metadata["foo"], "bar")
        self.assertEqual(updated.metadata["spam"], 1)

        update_project_input_2 = metadata.UpdateProjectInput(project.id, name + " Updated 2", metadata={"spam": None, "eggs": 2 })
        updated_2 = self._metadataclient.update_project(update_project_input_2)

        self.assertEqual(updated_2.name, name + " Updated 2")
        self.assertIsNone(updated_2.description)
        self.assertTrue("foo" not in updated_2.metadata)
        self.assertTrue("eggs" in updated_2.metadata)
        self.assertTrue("spam" in updated_2.metadata)
        self.assertIsNone(updated_2.metadata["spam"])
        self.assertEqual(updated_2.metadata["eggs"], 2)

    def test_project_members_is_ok(self):
        users = self._metadataclient.get_users()
        
        self.assertTrue(users)
        if(len(users) < 2):
            raise Exception("test_project_members_is_ok requires at least 2 users")

        project = self._create_project()
        self.assertIsNotNone(project)
        self._projects_to_delete.append(project.id)

        project_members = self._metadataclient.get_project_members(project.id)
        
        self.assertTrue(project_members)
        self.assertEqual(len(project_members), 1)

        first_member = project_members[0]

        second_user = [u for u in users if u.user_id != first_member.user_id][0]
        second_member = metadata.ProjectMember(second_user.user_id, metadata.ProjectMemberRole.CONTRIBUTOR)

        self._metadataclient.add_project_member(project.id, second_member)

        project_members = self._metadataclient.get_project_members(project.id)

        self.assertEqual(len(project_members), 2)

        self._metadataclient.remove_project_member(project.id, second_member.user_id)

        project_members = self._metadataclient.get_project_members(project.id)

        self.assertEqual(len(project_members), 1)

        row_version_output = self._metadataclient.set_project_members(project.id, [first_member, second_member])

        project_members = self._metadataclient.get_project_members(project.id)

        self.assertEqual(len(project_members), 2)

    def test_update_project_access_level_is_ok(self):
        project = self._create_project()

        self.assertIsNotNone(project)
        self._projects_to_delete.append(project.id)
        self.assertEqual(project.access_level, metadata.AccessLevel.SHARED)

        self._metadataclient.update_project_access_level(project.id, metadata.AccessLevel.PRIVATE)

        updated = self._metadataclient.get_project(project.id)
        self.assertEqual(updated.access_level, metadata.AccessLevel.PRIVATE)

    def test_subprojects_is_ok(self):
        project = self._create_project()
        self.assertIsNotNone(project)
        self._projects_to_delete.append(project.id)

        sub1input = metadata.SubprojectInput("Subproject 1")
        sub1 = self._metadataclient.create_subproject(project.id, sub1input)
        
        self.assertIsNotNone(sub1)

        sub2input = metadata.SubprojectInput("Subproject 2")
        sub2 = self._metadataclient.create_subproject(project.id, sub2input)

        self.assertIsNotNone(sub2)

        sub11input = metadata.SubprojectInput("Subproject 1 1", description="Subproject 1 of 1")
        sub11 = self._metadataclient.create_subproject(sub1.id, sub11input)
        
        self.assertIsNotNone(sub11)

        subprojects = self._metadataclient.list_subprojects(project.id)
        subs = [*subprojects]
        
        self.assertEqual(len(subs), 2)

        sub11path = self._metadataclient.get_project_path(sub11.id)
        path = "/".join([s.name for s in sub11path])

        self.assertEqual(path, project.name + "/Subproject 1/Subproject 1 1")

        self._metadataclient.move_project(sub11.id, sub2.id)

        subs1 = [*self._metadataclient.list_subprojects(sub1.id)]

        self.assertFalse(subs1)

        subs2 = [*self._metadataclient.list_subprojects(sub2.id)]
        
        self.assertEqual(len(subs2), 1)

    def test_prepare_hierarchy(self):
        project = self._create_project()    
        self.assertIsNotNone(project)
        self._projects_to_delete.append(project.id)

        actions = [
            metadata.PathAction.create_if_not_exists("foo", True),
            metadata.PathAction.create("foo/bar"),
            metadata.PathAction.create("foo/spam", True),
            metadata.PathAction.create("foo/spam/eggs"),
            metadata.PathAction.delete("foo/fred", True)
        ]

        hierarchy = self._metadataclient.prepare_hierarchy(project.id, actions)
        results = hierarchy.results
        self.assertTrue(results)
        self.assertEqual(4, len(results))
        self.assertEqual(2, len([r for r in results if r.datasetId is not None]))

    def test_get_sas_token_is_ok(self):
        project = self._create_project()
        self._projects_to_delete.append(project.id)
        token = self._metadataclient.get_sas_token_string(project.id, None, datetime.timedelta(hours=2))
        self.assertIsInstance(token, str)

    def test_get_service_url_is_ok(self):
        url = self._metadataclient.get_service_url("pubsub")
        self.assertIsInstance(url, str)

        

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2021 DHI A/S - DHI Water Environment Health 
# All rights reserved.
# 
# This code is licensed under the MIT License.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import itertools
import os
from .base.client import Contracts
from .generated.metadatagen import MetadataGenClientV2


class MetadataClientV2Helper:
    @classmethod
    def UploadFromFile(cls, filepath, clientv2: MetadataGenClientV2):
        uploadurlresponse = clientv2.GetUploadUrlV2()
        uploadurl = uploadurlresponse.Body.get("data")
        cls.UploadBlobFromFile(uploadurl, filepath)
        return uploadurl

    @staticmethod
    def UploadBlobFromFile(url, filepath):
        from azure.storage.blob import BlobClient
        blob = BlobClient.from_blob_url(url)
        with open(filepath, "rb") as f:
            blob.upload_blob(f)

    @staticmethod
    def DownloadBlobToFile(url, filepath):
        from azure.storage.blob import BlobClient
        blob = BlobClient.from_blob_url(url)
        download = blob.download_blob()
        if not filepath:
            filepath = os.path.basename(download.name)
            pos = filepath.find("_")
            if pos >= 0:
                filepath = filepath[pos+1:]
        with open(filepath, "wb") as f:
            download.readinto(f)

    @staticmethod
    def DeleteBlob(url):
        from azure.storage.blob import BlobClient
        blob = BlobClient.from_blob_url(url)
        if blob.exists():
            blob.delete_blob()


class MetadataClientV2Contracts(Contracts):
    @classmethod
    def PrepareUploadStagedFilesInput(cls, input=None, uploadurl=None, filename=None, destinationpath=None, createdestinationpathifnotexists=None):
        body = input.copy() if input else {}
        files = body.get("files")
        if not files:
            files = []
            cls.SetBodyField(body, "files", files)
        file1 = { "url": uploadurl, "filename": filename }
        files.append(file1)
        cls.SetBodyField(body, "destinationPath", destinationpath)
        cls.SetBodyField(body, "createDestinationPathIfNotExists", createdestinationpathifnotexists)
        return body


class MetadataClientV3Contracts(Contracts):
    @classmethod
    def PreparePrepareHierarchyInput(cls, input=None, actiontype=None, path=None, isfolder=None, defaultaccesslevel=None, sastokenexpiration=None):
        body = input.copy() if input else {}
        actions = body.get("actions")
        if not actions:
            actions = []
            body["actions"] = actions
        allfields = itertools.zip_longest(actiontype if actiontype else [], path if path else [], isfolder if isfolder else [])
        for t, p, isf in allfields:
            item = {}
            cls.SetBodyField(item, "type", t)
            cls.SetBodyField(item, "path", p)
            cls.SetBodyField(item, "isFolder", isf)
            actions.append(item)
        cls.SetBodyField(body, "defaultaccesslevel", defaultaccesslevel)
        cls.SetBodyField(body, "sastokenexpiration", sastokenexpiration)
        return body

    @classmethod
    def PrepareCreateProjectInput(cls, input=None, name=None, description=None, accesslevel=None, metadata=None, settings=None, members=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "name", name)
        cls.SetBodyField(body, "description", description)
        cls.SetBodyField(body, "accesslevel", accesslevel)
        cls.SetBodyField(body, "metadata", metadata, {})
        cls.SetBodyField(body, "settings", settings, {})
        cls.SetBodyField(body, "members", members, [])
        return body

    @classmethod
    def PrepareUpdateProjectInput(cls, input=None, projectid=None, name=None, description=None, metadata=None, settings=None, members=None, rowversion=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "id", projectid)
        cls.SetBodyField(body, "name", name)
        cls.SetBodyField(body, "description", description)
        cls.SetBodyField(body, "metadata", metadata, {})
        cls.SetBodyField(body, "settings", settings, {})
        cls.SetBodyField(body, "members", members, [])
        cls.SetBodyField(body, "rowversion", rowversion)
        return body

    @classmethod
    def PrepareConversionInput(cls, input=None, projectid=None, uploadurl=None, originalfilename=None, name=None, description=None, metadata=None, properties=None, readername=None, writername=None, readerparameters=None, writerparameters=None, transformations=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "projectId", projectid)
        cls.SetBodyField(body, "uploadUrl", uploadurl)
        cls.SetBodyField(body, "originalFilename", originalfilename)
        odd = body.get("outputDatasetData")
        if not odd:
            odd = {}
            cls.SetBodyField(body, "outputDatasetData", odd)
        cls.SetBodyField(odd, "name", name)
        cls.SetBodyField(odd, "description", description)
        cls.SetBodyField(odd, "metadata", metadata, {})
        cls.SetBodyField(odd, "properties", properties, {})
        cls.SetBodyField(body, "readerName", readername)
        cls.SetBodyField(body, "writerName", writername)
        cls.SetBodyField(body, "readerParameters", readerparameters, [])
        cls.SetBodyField(body, "writerParameters", writerparameters, [])
        cls.SetBodyField(body, "transformations", transformations, [])
        return body

    @classmethod
    def PrepareUpdateInput(cls, input=None, uploadurl=None, originalfilename=None, readername=None, writername=None, readerparameters=None, writerparameters=None, transformations=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "uploadUrl", uploadurl)
        cls.SetBodyField(body, "originalFilename", originalfilename)
        cls.SetBodyField(body, "readerName", readername)
        cls.SetBodyField(body, "writerName", writername)
        cls.SetBodyField(body, "readerParameters", readerparameters, [])
        cls.SetBodyField(body, "writerParameters", writerparameters, [])
        cls.SetBodyField(body, "transformations", transformations, [])
        return body

    @classmethod
    def PrepareAppendInput(cls, input=None, uploadurl=None, originalfilename=None, readername=None, writername=None, readerparameters=None, writerparameters=None, transformations=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "uploadUrl", uploadurl)
        cls.SetBodyField(body, "originalFilename", originalfilename)
        cls.SetBodyField(body, "readerName", readername)
        cls.SetBodyField(body, "writerName", writername)
        cls.SetBodyField(body, "readerParameters", readerparameters, [])
        cls.SetBodyField(body, "writerParameters", writerparameters, [])
        cls.SetBodyField(body, "transformations", transformations, [])
        return body

    @classmethod
    def PrepareDownloadDatasetInput(cls, input=None, dformat=None, srid=None, arguments=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "format", dformat)
        cls.SetBodyField(body, "srid", srid)
        cls.SetBodyField(body, "arguments", arguments, {})
        return body
    
    @classmethod
    def PrepareDownloadConvertDatasetInput(cls, input=None, targetfilename=None, readername=None, writername=None, readerparameters=None, writerparameters=None, transformations=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "targetFileName", targetfilename)
        cls.SetBodyField(body, "readerName", readername)
        cls.SetBodyField(body, "writerName", writername)
        cls.SetBodyField(body, "readerParameters", readerparameters, [])
        cls.SetBodyField(body, "writerParameters", writerparameters, [])
        cls.SetBodyField(body, "transformations", transformations, [])
        return body

    @classmethod
    def PrepareUploadInput(cls, input=None, projectid=None, inputformat=None, appenddatasetid=None, uploadurl=None, filename=None, srid=None, arguments=None, destinations=None, name=None, description=None, metadata=None, properties=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "projectId", projectid)
        cls.SetBodyField(body, "format", inputformat)
        cls.SetBodyField(body, "appenddatasetid", appenddatasetid)
        cls.SetBodyField(body, "uploadUrl", uploadurl)
        cls.SetBodyField(body, "filename", filename)
        cls.SetBodyField(body, "srid", srid)
        cls.SetBodyField(body, "arguments", arguments, {})
        cls.SetBodyField(body, "destinations", destinations, [])
        dsid = body.get("datasetImportData")
        if not dsid:
            dsid = {}
            cls.SetBodyField(body, "datasetImportData", dsid)
        cls.SetBodyField(dsid, "name", name)
        cls.SetBodyField(dsid, "description", description)
        cls.SetBodyField(dsid, "metadata", metadata, {})
        cls.SetBodyField(dsid, "properties", properties, {})
        return body

    @classmethod
    def PrepareCreateProjectMemberInput(cls, input=None, userid=None, role=None):
        body = input.copy() if input else {}
        cls.SetBodyField(body, "userid", userid)
        cls.SetBodyField(body, "role", role)
        return body


if __name__ == '__main__':
    print(__file__)
    print(dir())



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV2
from dhi.platform.fmt import Format

def initParser(parser):
    parser.add_argument("--filter", choices=["All", "File", "Dedicated"], help="Filter")

def main():
    args = ClientArgs.ParsePlatform(description="List readers", init=initParser)
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv2 = MetadataGenClientV2(**vars(args))

    response = clientv2.GetWritersListV2(args.filter)

    tablefmt = "{!s:32}\t{!s:16}\t{}"
    tablefields = ["name", "datasetFormat", "description"]
    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.enginehelper import EnginesClientV2Contracts
from dhi.platform.fmt import Format
from dhi.platform.generated.enginegen import EngineGenClientV2

def __initParser(parser):
    parser.add_argument("--sortby", help="Sort by field", choices=["ExecutionId", "Status", "ProjectId", "CreatedAt", "StartedAt", "UpdatedAt", "FinishedAt", "EngineName", "PoolType", "NodeCount", "TotalNumberOfSetups", "ScenarioName"])
    parser.add_argument("--asc", dest="sortorder", help="Use for ascending order", action="store_const", const="Asc")
    parser.add_argument("--desc", dest="sortorder", help="Use for descending order", action="store_const", const="Desc")
    parser.add_argument("--starttime", help="Start time filter")
    parser.add_argument("--endtime", help="End time filter")
    parser.add_argument("--scenarioname", help="Scenario name filter")
    parser.add_argument("--status", help="Status filter", choices=["Pending", "SettingUpComputeResources", "EvaluatingInputSize", "DownloadingInputFiles", "InProgress", "UploadingResults", "Success", "Failure", "Cancelling", "Cancelled", "Deleting"], nargs="+")
    parser.add_argument("--project", help="Project filter", nargs="+")
    parser.add_argument("--limit", type=int, help="Limit output size in one call")

def main():
    args = ClientArgs.ParseForProject(description="Get list of my execution runs", init=__initParser)
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv2 = EngineGenClientV2(**vars(args))

    def getResponses():
        cursor = None
        nextlink = None
        while True:
            if cursor:
                response = clientv2.GetMyExecutions(args.projectid, cursor=cursor)
            else:
                response = clientv2.GetMyExecutions(args.projectid, args.sortby, args.sortorder, args.starttime, args.endtime, args.scenarioname, args.status, args.project, args.limit, cursor=cursor)
            yield response
            cursor = response.Body.get("cursor")
            nextlink = response.Body.get("@nextLink")
            listobjdata = response.Body.get("data")
            if (not cursor and not nextlink) or not listobjdata:
                break
    responses = getResponses()

    def getList(r):
        return [EnginesClientV2Contracts.SetComputedDuration(EnginesClientV2Contracts.SetComputedPreparation(x)) for x in r.Body.get("data")]
    tablefmt = "{!s:36}\t{!s:25}\t{!s:28}\t{!s:14}\t{!s:14}\t{!s:9}\t{}"
    tablefields = ["executionId", "status", "createdAt", "preparation", "duration", "nodeCount", "engines"]
    Format.FormatResponses(responses, getList, args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("services", metavar="service", help="Service name", nargs="+")

def main():
    args = ClientArgs.ParsePlatform(description="Get service base URL", init=__initParser, defaultformat="yaml")
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    responses = (clientv3.GetServiceUrlV3(name) for name in args.services)

    Format.FormatResponses(responses, lambda r: r.Body, args.format)

if __name__ == '__main__':
    main()



# Copyright (c) 2021 DHI A/S - DHI Water Environment Health 
# All rights reserved.
# 
# This code is licensed under the MIT License.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import codecs
import pickle
import platform
import uuid
from azure.identity import InteractiveBrowserCredential, DeviceCodeCredential, TokenCachePersistenceOptions
from collections import namedtuple
from types import SimpleNamespace
from dhi.platform.base.exceptions import MikeCloudException
from dhi.platform.config import ClientConfig
from abc import ABC, abstractmethod


#import adal, azure.common.credentials, pickle
#from azure.identity import DefaultAzureCredential, InteractiveBrowserCredential, SharedTokenCacheCredential
#from msal_interactive_token_acquirer import MsalInteractiveTokenAcquirer
#from typing import NamedTuple


class ClientAuthentication:
    @classmethod
    def RefreshToken(cls, environment, accesstoken):
        info = cls.__GetAuthInfo(environment, "meta")
        if info:
            scopes = [f"{info.appid}/user_impersonation"]
            authsaved = accesstoken.get("authenticationrecord")
            if authsaved:
                options = _AuthenticationHelper.GetDefaultTokenCacheOptions()
                auth = pickle.loads(codecs.decode(authsaved.encode(), "base64"))
                credential = _AuthenticationHelper.GetDefaultInteractiveCredential(authentication_record=auth, cache_persistence_options=options)
                try:
                    token = credential.get_token(*scopes)
                except:
                    token = None
                if token:
                    auth = credential.authenticate(scopes=scopes)
                    authb64 = codecs.encode(pickle.dumps(auth), "base64").decode()
                    #return (cls.__AccessToken(token.token, token.expires_on, auth.username, authb64)._asdict(), token.token)
                    return (cls.__AccessToken(None, token.expires_on, auth.username, authb64)._asdict(), token.token)
        return (None, None)

    @classmethod
    def AcquireTokenInteractively(cls, environment):
        info = cls.__GetAuthInfo(environment, "meta")
        if info:
            #authorityUrl = f"https://login.microsoftonline.com/{tenant}"
            #context = adal.AuthenticationContext(authorityUrl, validate_authority=tenant != "adfs")
            #resource = "00000002-0000-0000-c000-000000000000"
            #clientSecret = ""
            #return context.acquire_token_with_client_credentials(resource, clientId, clientSecret)
            #return ontext.acquire_token(resource, userId, clientId)
            #
            #scopes = [f"{metaResourceId}/user_impersonation"]
            #msal = MsalInteractiveTokenAcquirer(tenant, clientId, scopes)
            #msal.acquire_token_interactively()
            #return msal.access_token()
            #
            #azure.common.credentials.InteractiveCredentials()
            #
            authority = "login.microsoftonline.com"
            scopes = [f"{info.appid}/user_impersonation"]
            #cacheCredential = SharedTokenCacheCredential(authority=authority, tenant_id=tenant)
            #token = cacheCredential.get_token(*scopes)
            options = _AuthenticationHelper.GetDefaultTokenCacheOptions()
            credential = _AuthenticationHelper.GetDefaultInteractiveCredential(authority=authority, tenant_id=info.tenant, client_id=info.clientid, cache_persistence_options=options)
            try:
                token = credential.get_token(*scopes)
            except:
                token = None
            if token:
                auth = credential.authenticate(scopes=scopes)
                authb64 = codecs.encode(pickle.dumps(auth), "base64").decode()
                #return (cls.__AccessToken(token.token, token.expires_on, auth.username, authb64)._asdict(), token.token)
                return (cls.__AccessToken(None, token.expires_on, auth.username, authb64)._asdict(), token.token)
        return (None, None)

    @classmethod
    def __GetAuthInfo(cls, environment, app):
        if env := cls.__config.get(environment):
            return SimpleNamespace(tenant=env.tenant, clientid=env.clientid, appid=env.apps.get(app))
        return None

    __AccessToken = namedtuple("__AccessToken", ["accesstoken", "expireson", "username", "authenticationrecord"])

    __EnvironmentAuthInfo = namedtuple("__EnvironmentAuthInfo", ["environment", "tenant", "clientid", "apps"])

    __config = {
        # PROD
        "prod": __EnvironmentAuthInfo("prod", "dhigroupext.onmicrosoft.com", "aa00293d-f5dd-4ab6-a3c8-28f894149572", {"meta": "0dc51ba3-5c5e-4703-ba89-322411a51197"}),
        # PREPROD ?
        "preprod": __EnvironmentAuthInfo("preprod", "dhigrouppreprod.onmicrosoft.com", "aa00293d-f5dd-4ab6-a3c8-28f894149572", {"meta": "0dc51ba3-5c5e-4703-ba89-322411a51197"}),
        # TEST
        "test": __EnvironmentAuthInfo("test", "dhigrouptest.onmicrosoft.com", "189228fc-e229-4f7d-99c2-ec5161f5d31a", {"meta": "14583d65-56bc-4bff-9ace-7e80cbdc8944"}),
        # DEV0 - dhi.core.dev.uitest app.
        "DEV0": __EnvironmentAuthInfo("DEV0", "dhigroupdev.onmicrosoft.com", "ffa42221-f5e6-47ac-8ab2-65ba5b23a272", {"meta": "5122f7bd-0a44-4bea-8043-d6da1b52b49b"}),
        # DEV0EU - dhi.core.dev.uitest app.
        "DEV0EU": __EnvironmentAuthInfo("DEV0EU", "dhigroupdev.onmicrosoft.com", "ffa42221-f5e6-47ac-8ab2-65ba5b23a272", {"meta": "5122f7bd-0a44-4bea-8043-d6da1b52b49b"}),
        # DEV - dhi.core.dev.uitest app.
        "DEV": __EnvironmentAuthInfo("DEV", "dhigroupdev.onmicrosoft.com", "ffa42221-f5e6-47ac-8ab2-65ba5b23a272", {"meta": "425386dd-16e2-4225-a966-dce5d6a07402"}),
        # DEV - external AD
        "dev": __EnvironmentAuthInfo("dev", "dhigroupdevext.onmicrosoft.com", "68803cd3-78d5-4343-a135-d4402491babd", {"meta": "cbc2864b-b8c2-465b-a440-9f5b32957bc8"}),
        # DEV0 - external AD
        "dev0": __EnvironmentAuthInfo("dev0", "dhigroupdevext.onmicrosoft.com", "68803cd3-78d5-4343-a135-d4402491babd", {"meta": "7c8997e4-7d1d-4b16-a562-c6165f99762a"}),
        # DEV0EU - external AD
        "dev0eu": __EnvironmentAuthInfo("dev0eu", "dhigroupdevext.onmicrosoft.com", "68803cd3-78d5-4343-a135-d4402491babd", {"meta": "7c8997e4-7d1d-4b16-a562-c6165f99762a"})
    }


def GetAuthInfo(environment = "local"):
    if environment == "prod":
        # PROD
        tenant = "dhigroupext.onmicrosoft.com"
        clientId = "aa00293d-f5dd-4ab6-a3c8-28f894149572"
        metaResourceId = "0dc51ba3-5c5e-4703-ba89-322411a51197"
    elif environment == "preprod":
        # PREPROD ?
        tenant = "dhigroupext.onmicrosoft.com"
        clientId = "aa00293d-f5dd-4ab6-a3c8-28f894149572"
        metaResourceId = "0dc51ba3-5c5e-4703-ba89-322411a51197"
    elif environment == "test":
        # TEST
        tenant = "dhigrouptest.onmicrosoft.com"
        clientId = "189228fc-e229-4f7d-99c2-ec5161f5d31a"
        metaResourceId = "14583d65-56bc-4bff-9ace-7e80cbdc8944"
    elif environment == "DEV0" or environment == "DEV0EU":
        # DEV0 - dhi.core.dev.uitest app.
        tenant = "dhigroupdev.onmicrosoft.com"
        clientId = "ffa42221-f5e6-47ac-8ab2-65ba5b23a272"
        metaResourceId = "5122f7bd-0a44-4bea-8043-d6da1b52b49b"
    elif environment == "DEV":
        # DEV - dhi.core.dev.uitest app.
        tenant = "dhigroupdev.onmicrosoft.com"
        clientId = "ffa42221-f5e6-47ac-8ab2-65ba5b23a272"
        metaResourceId = "425386dd-16e2-4225-a966-dce5d6a07402"
    elif environment == "dev":
        # DEV - external AD
        tenant = "dhigroupdevext.onmicrosoft.com"
        clientId = "68803cd3-78d5-4343-a135-d4402491babd"
        metaResourceId = "cbc2864b-b8c2-465b-a440-9f5b32957bc8"
    elif environment == "dev0" or environment == "dev0eu":
        # DEV0 - external AD
        tenant = "dhigroupdevext.onmicrosoft.com"
        clientId = "68803cd3-78d5-4343-a135-d4402491babd"
        metaResourceId = "7c8997e4-7d1d-4b16-a562-c6165f99762a"
    else:
        return None, None, None
    return tenant, clientId, metaResourceId

def AcquireTokenInteractively(environment):
    tenant, clientId, metaResourceId = GetAuthInfo(environment)
    if tenant and clientId and metaResourceId:
        #authorityUrl = f"https://login.microsoftonline.com/{tenant}"
        #context = adal.AuthenticationContext(authorityUrl, validate_authority=tenant != "adfs")
        #resource = "00000002-0000-0000-c000-000000000000"
        #clientSecret = ""
        #return context.acquire_token_with_client_credentials(resource, clientId, clientSecret)
        #return ontext.acquire_token(resource, userId, clientId)

        #scopes = [f"{metaResourceId}/user_impersonation"]
        #msal = MsalInteractiveTokenAcquirer(tenant, clientId, scopes)
        #msal.acquire_token_interactively()
        #return msal.access_token()

        #azure.common.credentials.InteractiveCredentials()

        authority = "login.microsoftonline.com"
        scopes = [f"{metaResourceId}/user_impersonation"]

        #cacheCredential = SharedTokenCacheCredential(authority=authority, tenant_id=tenant)
        #token = cacheCredential.get_token(*scopes)
        credential = _AuthenticationHelper.GetDefaultInteractiveCredential(authority=authority, tenant_id=tenant, client_id=clientId)
        token = credential.get_token(*scopes)
        #auth = credential.authenticate(scopes=scopes)
        return token

    return None

class MikeCloudIdentity(ABC):

    @abstractmethod
    def get_parameters(self):
        return {}

class ApiKeyIdentity(MikeCloudIdentity):
    """Identity to log in with open api key"""

    def __init__(self, customer_id, apikey, environment=None):
        super().__init__()

        try:
            id = uuid.UUID(apikey)
        except ValueError:
            parameter_name = f"{apikey=}".split("=")[0]
            raise MikeCloudException(f"Parameter {parameter_name} must be a uuid, not {apikey}")

        self._customer_id = customer_id
        self._apikey = apikey
        self._environment = environment if environment else "prod"

    @property
    def apikey(self):
        return self._apikey

    @property
    def customer_id(self):
        return self._customer_id

    @property
    def environment(self):
        return self._environment

    def get_parameters(self):
        data = {
            'apikey': self.apikey,
            'environment': self.environment
        }

        if self.customer_id:
            data["customer_id"] = self.customer_id

        return data


class InteractiveIdentity(MikeCloudIdentity):
    """Identity for interactive user authentication"""

    _token = None
    _environment = None

    def __init__(self, environment=None, forcelogin=True) -> None:
        super().__init__()
        self._environment = environment if environment else "prod"
        self._token = ClientConfig.AcquireAccessToken(self._environment, forcelogin=forcelogin)
    
    @property
    def token(self):
        if self._token:
            return self._token
        else:
            raise MikeCloudException("Interactive token does not exist yet")

    @property
    def environment(self):
        return self._environment

    def get_parameters(self):
        return {
            'accesstoken': self.token,
            'environment': self.environment
        }


class _AuthenticationHelper:
    @staticmethod
    def GetDefaultInteractiveCredential(**kwargs):
        if platform.system() == 'Linux':
            return DeviceCodeCredential(**kwargs)
        return InteractiveBrowserCredential(**kwargs)

    @staticmethod
    def GetDefaultTokenCacheOptions(**kwargs):
        allowunencryptedstorage = platform.system() == 'Linux'
        return TokenCachePersistenceOptions(name="dhi-platform-sdk-py", allow_unencrypted_storage=allowunencryptedstorage)


if __name__ == '__main__':
    print(__file__)
    print(dir())



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def main():
    args = ClientArgs.ParseForProject(description="Get project path")
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    response = clientv3.GetProjectPathV3(args.projectid)

    tablefmt = "{!s:32}\t{!s:26}\t{!s:12}\t{!s:16}\t{}"
    tablefields = ["id", "name", "accessLevel", "inheritsMembers", "efectiveUserRole"]
    Format.FormatResponse(response, lambda r: r.Body.get("data"), args.format, tablefmt, tablefields)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def main():
    args = ClientArgs.ParseForProjectList(description="Get project capabilities", defaultformat="yaml")
    ClientConfig.UpdatePlatformFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    responses = (clientv3.GetProjectCapabilitiesV3(id) for id in args.projectids)

    tablefields = ["canEdit", "canEditAccessLevel", "canDelete", "canGrantAccess", "canCreateContent", "canListContent", "canUpdateContent", "canDeleteContent", "canReadContent"]
    Format.FormatResponses(responses, lambda r: r.Body, args.format, None, tablefields)

if __name__ == '__main__':
    main()



# Generated using openapi2py.py
# openapi2py.py "--dhiservice" "pubsub" "--classname" "PubSubGenClientV" "-r" "projectid" "-i" "datasetid=id" "--out" "C:\work\devops\mike-platform-sdk-py\src\dhi\platform\generated\pubsubgen.py" "https://apispec-mike-platform-dev0.eu.mike-cloud-dev0.com/pubsub/v1"
# 2022-01-13 19:03:34.024669Z
from typing import Any, Dict, List, NewType, Type, TypeVar, Union
from enum import Enum
import attr
from ..base.client import DataContract, PlatformClient, Response


# https://apispec-mike-platform-dev0.eu.mike-cloud-dev0.com/pubsub/v1
# DHI Water Data Eventing API - Version 1
# API for publishing and subscribing to events
# 1

EventDataV1Type = TypeVar("EventDataV1Type", bound="EventDataV1")

@attr.s(auto_attribs=True)
class EventDataV1(DataContract):
    """See https://github.com/cloudevents/spec/blob/master/spec.md

    """
    source: str = None
    type: str = None
    subject: str = None
    dataSchema: str = None
    data: None = None
    time: str = None
    operationId: str = None
    resourceId: str = None
    isTransient: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: EventDataV1Type, src_dict: Dict[str, Any]) -> EventDataV1Type:
        obj = EventDataV1()
        obj.load_dict(src_dict)
        return obj

ProblemDetailsV1Type = TypeVar("ProblemDetailsV1Type", bound="ProblemDetailsV1")

@attr.s(auto_attribs=True)
class ProblemDetailsV1(DataContract):
    type: str = None
    title: str = None
    status: int = None
    detail: str = None
    instance: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ProblemDetailsV1Type, src_dict: Dict[str, Any]) -> ProblemDetailsV1Type:
        obj = ProblemDetailsV1()
        obj.load_dict(src_dict)
        return obj

ValidationProblemDetailsV1Type = TypeVar("ValidationProblemDetailsV1Type", bound="ValidationProblemDetailsV1")

@attr.s(auto_attribs=True)
class ValidationProblemDetailsV1(ProblemDetailsV1):
    errors: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = ProblemDetailsV1.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ValidationProblemDetailsV1Type, src_dict: Dict[str, Any]) -> ValidationProblemDetailsV1Type:
        obj = ValidationProblemDetailsV1()
        obj.load_dict(src_dict)
        return obj

class PubSubGenClientV1(PlatformClient):
    def __init__(self, inspectFnc=PlatformClient.DefaultInspectFnc, **kwargs):
        super().__init__(inspectFnc, includeheaders=PlatformClient.GetServiceHeaders("pubsub"), **kwargs)

    def CreateEvent(self, projectid, body) -> Response:
        """Create cloud event

        Event
        POST /api/eventing/event
        """
        return self.PostRequest("/api/eventing/event", body, None, api_version="1", projectid=projectid)



# Generated using openapi2py.py
# openapi2py.py "--dhiservice" "raw" "--classname" "RawGenClientV" "-n" "2" "-r" "projectid" "-i" "datasetid=id" "--out" "C:\work\devops\mike-platform-sdk-py\src\dhi\platform\generated\rawgen.py" "https://apispec-mike-platform-dev0.eu.mike-cloud-dev0.com/raw/v2"
# 2022-01-13 19:04:07.997380Z
from typing import Any, Dict, List, NewType, Type, TypeVar, Union
from enum import Enum
import attr
from ..base.client import DataContract, PlatformClient, Response


# https://apispec-mike-platform-dev0.eu.mike-cloud-dev0.com/raw/v2
#  - Version 2
# 
# 2

GetFileChecksumOutputV2Type = TypeVar("GetFileChecksumOutputV2Type", bound="GetFileChecksumOutputV2")

@attr.s(auto_attribs=True)
class GetFileChecksumOutputV2(DataContract):
    id: str = None
    name: str = None
    checksum: str = None
    alg: str = None
    lastModified: str = None
    size: int = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetFileChecksumOutputV2Type, src_dict: Dict[str, Any]) -> GetFileChecksumOutputV2Type:
        obj = GetFileChecksumOutputV2()
        obj.load_dict(src_dict)
        return obj

UploadBulkFileInputV2Type = TypeVar("UploadBulkFileInputV2Type", bound="UploadBulkFileInputV2")

@attr.s(auto_attribs=True)
class UploadBulkFileInputV2(DataContract):
    id: str = None
    sasToken: str = None
    name: str = None
    url: str = None
    lastModified: str = None
    size: int = None
    forceCopy: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: UploadBulkFileInputV2Type, src_dict: Dict[str, Any]) -> UploadBulkFileInputV2Type:
        obj = UploadBulkFileInputV2()
        obj.load_dict(src_dict)
        return obj

GetCopyFileStatusInputV2Type = TypeVar("GetCopyFileStatusInputV2Type", bound="GetCopyFileStatusInputV2")

@attr.s(auto_attribs=True)
class GetCopyFileStatusInputV2(DataContract):
    id: str = None
    copyOperationId: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetCopyFileStatusInputV2Type, src_dict: Dict[str, Any]) -> GetCopyFileStatusInputV2Type:
        obj = GetCopyFileStatusInputV2()
        obj.load_dict(src_dict)
        return obj

GetCopyFilesStatusInputV2Type = TypeVar("GetCopyFilesStatusInputV2Type", bound="GetCopyFilesStatusInputV2")

@attr.s(auto_attribs=True)
class GetCopyFilesStatusInputV2(DataContract):
    datasets: List[GetCopyFileStatusInputV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetCopyFilesStatusInputV2Type, src_dict: Dict[str, Any]) -> GetCopyFilesStatusInputV2Type:
        obj = GetCopyFilesStatusInputV2()
        obj.load_dict(src_dict)
        return obj

GetDownloadDatasetOutputV2Type = TypeVar("GetDownloadDatasetOutputV2Type", bound="GetDownloadDatasetOutputV2")

@attr.s(auto_attribs=True)
class GetDownloadDatasetOutputV2(DataContract):
    id: str = None
    name: str = None
    url: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetDownloadDatasetOutputV2Type, src_dict: Dict[str, Any]) -> GetDownloadDatasetOutputV2Type:
        obj = GetDownloadDatasetOutputV2()
        obj.load_dict(src_dict)
        return obj

GetDownloadDatasetOutputCollectionResponseV2Type = TypeVar("GetDownloadDatasetOutputCollectionResponseV2Type", bound="GetDownloadDatasetOutputCollectionResponseV2")

@attr.s(auto_attribs=True)
class GetDownloadDatasetOutputCollectionResponseV2(DataContract):
    data: List[GetDownloadDatasetOutputV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetDownloadDatasetOutputCollectionResponseV2Type, src_dict: Dict[str, Any]) -> GetDownloadDatasetOutputCollectionResponseV2Type:
        obj = GetDownloadDatasetOutputCollectionResponseV2()
        obj.load_dict(src_dict)
        return obj

DownloadOutputV2Type = TypeVar("DownloadOutputV2Type", bound="DownloadOutputV2")

@attr.s(auto_attribs=True)
class DownloadOutputV2(DataContract):
    datasetName: str = None
    url: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: DownloadOutputV2Type, src_dict: Dict[str, Any]) -> DownloadOutputV2Type:
        obj = DownloadOutputV2()
        obj.load_dict(src_dict)
        return obj

MoveStagedFileInputV2Type = TypeVar("MoveStagedFileInputV2Type", bound="MoveStagedFileInputV2")

@attr.s(auto_attribs=True)
class MoveStagedFileInputV2(DataContract):
    stagedFileUrl: str = None
    datasetName: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: MoveStagedFileInputV2Type, src_dict: Dict[str, Any]) -> MoveStagedFileInputV2Type:
        obj = MoveStagedFileInputV2()
        obj.load_dict(src_dict)
        return obj

ProblemDetailsV2Type = TypeVar("ProblemDetailsV2Type", bound="ProblemDetailsV2")

@attr.s(auto_attribs=True)
class ProblemDetailsV2(DataContract):
    type: str = None
    title: str = None
    status: int = None
    detail: str = None
    instance: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ProblemDetailsV2Type, src_dict: Dict[str, Any]) -> ProblemDetailsV2Type:
        obj = ProblemDetailsV2()
        obj.load_dict(src_dict)
        return obj

ValidationProblemDetailsV2Type = TypeVar("ValidationProblemDetailsV2Type", bound="ValidationProblemDetailsV2")

@attr.s(auto_attribs=True)
class ValidationProblemDetailsV2(ProblemDetailsV2):
    errors: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = ProblemDetailsV2.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: ValidationProblemDetailsV2Type, src_dict: Dict[str, Any]) -> ValidationProblemDetailsV2Type:
        obj = ValidationProblemDetailsV2()
        obj.load_dict(src_dict)
        return obj

UploadBulkFilesInputV2Type = TypeVar("UploadBulkFilesInputV2Type", bound="UploadBulkFilesInputV2")

@attr.s(auto_attribs=True)
class UploadBulkFilesInputV2(DataContract):
    datasets: List[UploadBulkFileInputV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: UploadBulkFilesInputV2Type, src_dict: Dict[str, Any]) -> UploadBulkFilesInputV2Type:
        obj = UploadBulkFilesInputV2()
        obj.load_dict(src_dict)
        return obj

GetFileChecksumOutputCollectionResponseV2Type = TypeVar("GetFileChecksumOutputCollectionResponseV2Type", bound="GetFileChecksumOutputCollectionResponseV2")

@attr.s(auto_attribs=True)
class GetFileChecksumOutputCollectionResponseV2(DataContract):
    data: List[GetFileChecksumOutputV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetFileChecksumOutputCollectionResponseV2Type, src_dict: Dict[str, Any]) -> GetFileChecksumOutputCollectionResponseV2Type:
        obj = GetFileChecksumOutputCollectionResponseV2()
        obj.load_dict(src_dict)
        return obj

UploadOutputV2Type = TypeVar("UploadOutputV2Type", bound="UploadOutputV2")

@attr.s(auto_attribs=True)
class UploadOutputV2(DataContract):
    id: str = None
    url: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: UploadOutputV2Type, src_dict: Dict[str, Any]) -> UploadOutputV2Type:
        obj = UploadOutputV2()
        obj.load_dict(src_dict)
        return obj

IsStagingUrlInputV2Type = TypeVar("IsStagingUrlInputV2Type", bound="IsStagingUrlInputV2")

@attr.s(auto_attribs=True)
class IsStagingUrlInputV2(DataContract):
    url: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: IsStagingUrlInputV2Type, src_dict: Dict[str, Any]) -> IsStagingUrlInputV2Type:
        obj = IsStagingUrlInputV2()
        obj.load_dict(src_dict)
        return obj

CopyOperationOutputV2Type = TypeVar("CopyOperationOutputV2Type", bound="CopyOperationOutputV2")

@attr.s(auto_attribs=True)
class CopyOperationOutputV2(DataContract):
    id: str = None
    hasCompleted: str = None
    copiedBytes: int = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: CopyOperationOutputV2Type, src_dict: Dict[str, Any]) -> CopyOperationOutputV2Type:
        obj = CopyOperationOutputV2()
        obj.load_dict(src_dict)
        return obj

GetCopyFilesStatusOutputV2Type = TypeVar("GetCopyFilesStatusOutputV2Type", bound="GetCopyFilesStatusOutputV2")

@attr.s(auto_attribs=True)
class GetCopyFilesStatusOutputV2(DataContract):
    id: str = None
    name: str = None
    copyOperation: CopyOperationOutputV2 = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetCopyFilesStatusOutputV2Type, src_dict: Dict[str, Any]) -> GetCopyFilesStatusOutputV2Type:
        obj = GetCopyFilesStatusOutputV2()
        obj.load_dict(src_dict)
        return obj

GetCopyFilesStatusOutputCollectionResponseV2Type = TypeVar("GetCopyFilesStatusOutputCollectionResponseV2Type", bound="GetCopyFilesStatusOutputCollectionResponseV2")

@attr.s(auto_attribs=True)
class GetCopyFilesStatusOutputCollectionResponseV2(DataContract):
    data: List[GetCopyFilesStatusOutputV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetCopyFilesStatusOutputCollectionResponseV2Type, src_dict: Dict[str, Any]) -> GetCopyFilesStatusOutputCollectionResponseV2Type:
        obj = GetCopyFilesStatusOutputCollectionResponseV2()
        obj.load_dict(src_dict)
        return obj

StagingUrlOutputV2Type = TypeVar("StagingUrlOutputV2Type", bound="StagingUrlOutputV2")

@attr.s(auto_attribs=True)
class StagingUrlOutputV2(DataContract):
    url: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: StagingUrlOutputV2Type, src_dict: Dict[str, Any]) -> StagingUrlOutputV2Type:
        obj = StagingUrlOutputV2()
        obj.load_dict(src_dict)
        return obj

StagingUrlOutputCollectionResponseV2Type = TypeVar("StagingUrlOutputCollectionResponseV2Type", bound="StagingUrlOutputCollectionResponseV2")

@attr.s(auto_attribs=True)
class StagingUrlOutputCollectionResponseV2(DataContract):
    data: List[StagingUrlOutputV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: StagingUrlOutputCollectionResponseV2Type, src_dict: Dict[str, Any]) -> StagingUrlOutputCollectionResponseV2Type:
        obj = StagingUrlOutputCollectionResponseV2()
        obj.load_dict(src_dict)
        return obj

UploadBulkFileOutputV2Type = TypeVar("UploadBulkFileOutputV2Type", bound="UploadBulkFileOutputV2")

@attr.s(auto_attribs=True)
class UploadBulkFileOutputV2(DataContract):
    id: str = None
    name: str = None
    size: int = None
    copyOperation: CopyOperationOutputV2 = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: UploadBulkFileOutputV2Type, src_dict: Dict[str, Any]) -> UploadBulkFileOutputV2Type:
        obj = UploadBulkFileOutputV2()
        obj.load_dict(src_dict)
        return obj

UploadBulkFileOutputCollectionResponseV2Type = TypeVar("UploadBulkFileOutputCollectionResponseV2Type", bound="UploadBulkFileOutputCollectionResponseV2")

@attr.s(auto_attribs=True)
class UploadBulkFileOutputCollectionResponseV2(DataContract):
    data: List[UploadBulkFileOutputV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: UploadBulkFileOutputCollectionResponseV2Type, src_dict: Dict[str, Any]) -> UploadBulkFileOutputCollectionResponseV2Type:
        obj = UploadBulkFileOutputCollectionResponseV2()
        obj.load_dict(src_dict)
        return obj

StringResponseV2Type = TypeVar("StringResponseV2Type", bound="StringResponseV2")

@attr.s(auto_attribs=True)
class StringResponseV2(DataContract):
    data: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: StringResponseV2Type, src_dict: Dict[str, Any]) -> StringResponseV2Type:
        obj = StringResponseV2()
        obj.load_dict(src_dict)
        return obj

IsStagingUrlOutputV2Type = TypeVar("IsStagingUrlOutputV2Type", bound="IsStagingUrlOutputV2")

@attr.s(auto_attribs=True)
class IsStagingUrlOutputV2(DataContract):
    result: str = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: IsStagingUrlOutputV2Type, src_dict: Dict[str, Any]) -> IsStagingUrlOutputV2Type:
        obj = IsStagingUrlOutputV2()
        obj.load_dict(src_dict)
        return obj

GetDatasetBlockChecksumOutputV2Type = TypeVar("GetDatasetBlockChecksumOutputV2Type", bound="GetDatasetBlockChecksumOutputV2")

@attr.s(auto_attribs=True)
class GetDatasetBlockChecksumOutputV2(DataContract):
    checksum: str = None
    size: int = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetDatasetBlockChecksumOutputV2Type, src_dict: Dict[str, Any]) -> GetDatasetBlockChecksumOutputV2Type:
        obj = GetDatasetBlockChecksumOutputV2()
        obj.load_dict(src_dict)
        return obj

GetDatasetBlocksChecksumsOutputV2Type = TypeVar("GetDatasetBlocksChecksumsOutputV2Type", bound="GetDatasetBlocksChecksumsOutputV2")

@attr.s(auto_attribs=True)
class GetDatasetBlocksChecksumsOutputV2(DataContract):
    datasetId: str = None
    blocks: List[GetDatasetBlockChecksumOutputV2] = None
    __renamed = {  }
    def to_dict(self) -> Dict[str, Any]:
        return self.get_dictionary(self.get_renamed())
    def load_dict(self, src_dict: Dict[str, Any]) -> None:
        DataContract.load_from_directory(self, src_dict, self.get_renamed())
    @classmethod
    def get_renamed(cls) -> Dict[str, str]:
        result = DataContract.get_renamed().copy()
        result.update(cls.__renamed)
        return result
    @classmethod
    def from_dict(cls: GetDatasetBlocksChecksumsOutputV2Type, src_dict: Dict[str, Any]) -> GetDatasetBlocksChecksumsOutputV2Type:
        obj = GetDatasetBlocksChecksumsOutputV2()
        obj.load_dict(src_dict)
        return obj

class RawGenClientV2(PlatformClient):
    def __init__(self, inspectFnc=PlatformClient.DefaultInspectFnc, **kwargs):
        super().__init__(inspectFnc, includeheaders=PlatformClient.GetServiceHeaders("raw"), **kwargs)

    def GetFilesChecksums(self, projectid, id=None) -> Response:
        """Get checksums for selected or all files in a folder.

        FileSync
        GET /api/filesync/checksums
        """
        queryparams = self.GetQueryParams(id=id)
        return self.GetRequest("/api/filesync/checksums", queryparams, api_version="2", projectid=projectid, datasetid=id)

    def GetDownloadDatasets(self, projectid, id=None) -> Response:
        """Get download info for selected or all files in a folder.

        FileSync
        GET /api/filesync/prepare-download
        """
        queryparams = self.GetQueryParams(id=id)
        return self.GetRequest("/api/filesync/prepare-download", queryparams, api_version="2", projectid=projectid, datasetid=id)

    def UploadBulkFiles(self, projectid, body) -> Response:
        """Upload file datasets from the staging area or another storage.
        Files in the staging area moved, all other files are copied.
        Check the copy operations with "POST /api/filesync/upload/status".

        FileSync
        PUT /api/filesync/upload
        """
        return self.PutRequest("/api/filesync/upload", body, None, api_version="2", projectid=projectid)

    def GetCopyFilesStatus(self, projectid, body) -> Response:
        """Get upload copy operations status, copy operations started in "PUT /api/filesync/upload".

        FileSync
        POST /api/filesync/upload/status
        """
        return self.PostRequest("/api/filesync/upload/status", body, None, api_version="2", projectid=projectid)

    def GetDatasetBlocksChecksums(self, projectid, id) -> Response:
        """Get file blocks with checksums.

        FileSync
        GET /api/filesync/{id}/blocks
        """
        return self.GetRequest(f"/api/filesync/{id}/blocks", None, api_version="2", projectid=projectid, datasetid=id)

    def GetFileSasUrlV2(self, projectid, id, downloadfilename=None) -> Response:
        """Prepare download of data for dataset. When you call this service directly you need to provide a SAS token
        for the given dataset with read privilege to download data for a dataset.

        Raw
        GET /api/raw/dataset/{id}
        """
        queryparams = self.GetQueryParams(downloadFileName=downloadfilename)
        return self.GetRequest(f"/api/raw/dataset/{id}", queryparams, api_version="2", projectid=projectid, datasetid=id)

    def MoveStagedFileToDataset(self, projectid, body) -> Response:
        """Move data from staging storage to dataset.

        Staging
        POST /api/raw/move-staged-url
        """
        return self.PostRequest("/api/raw/move-staged-url", body, None, api_version="2", projectid=projectid)

    def GetDatasetDownloadUrl(self, projectid, datasetid, downloadfilename=None) -> Response:
        """Prepare download of data for dataset.

        Dataset
        GET /api/raw/prepare-download/{datasetId}
        """
        queryparams = self.GetQueryParams(downloadFileName=downloadfilename)
        return self.GetRequest(f"/api/raw/prepare-download/{datasetid}", queryparams, api_version="2", projectid=projectid)

    def GetDatasetUploadUrl(self, projectid, datasetname) -> Response:
        """Prepare upload of data for dataset.

        Dataset
        GET /api/raw/prepare-upload/{datasetName}
        """
        return self.GetRequest(f"/api/raw/prepare-upload/{datasetname}", None, api_version="2", projectid=projectid)

    def GetStagingUrlV2(self, projectid) -> Response:
        """Prepare staging blob storage for data upload.

        Staging
        GET /api/raw/staging-url
        """
        return self.GetRequest("/api/raw/staging-url", None, api_version="2", projectid=projectid)

    def IsStagingUrlV2(self, projectid, body) -> Response:
        """Check if given url is a staging url

        Staging
        POST /api/raw/staging-url
        """
        return self.PostRequest("/api/raw/staging-url", body, None, api_version="2", projectid=projectid)

    def GetStagingUrls(self, projectid, count) -> Response:
        """Prepare staging blob storages for data upload.

        Staging
        GET /api/raw/staging-urls
        """
        queryparams = self.GetQueryParams(count=count)
        return self.GetRequest("/api/raw/staging-urls", queryparams, api_version="2", projectid=projectid)



#!/usr/bin/env python
from dhi.platform.args import ClientArgs
from dhi.platform.config import ClientConfig
from dhi.platform.generated.metadatagen import MetadataGenClientV3
from dhi.platform.fmt import Format

def __initParser(parser):
    parser.add_argument("userids", metavar="userid", help="User id", nargs="+")

def main():
    args = ClientArgs.ParseForProject(description="Remove project members", init=__initParser, defaultformat="yaml")
    ClientConfig.UpdateProjectFromConfiguration(args)
    clientv3 = MetadataGenClientV3(**vars(args))

    responses = (clientv3.DeleteProjectMemberV3(id, args.projectid) for id in args.userids)

    Format.FormatResponses(responses, None, args.format)

if __name__ == '__main__':
    main()
